/* valaprojectgenerator.vala
 *
 * Copyright (C) 2007-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 * 	Andrea Del Signore <sejerpz@tin.it>
 */

#ifndef __GEN_PROJECT_VALAPROJECTGENERATOR_H__
#define __GEN_PROJECT_VALAPROJECTGENERATOR_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

G_BEGIN_DECLS


#define VALA_TYPE_PROJECT_TYPE (vala_project_type_get_type ())

#define VALA_TYPE_PROJECT_LICENSE (vala_project_license_get_type ())

#define VALA_TYPE_PROJECT_GENERATOR (vala_project_generator_get_type ())
#define VALA_PROJECT_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_PROJECT_GENERATOR, ValaProjectGenerator))
#define VALA_PROJECT_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_PROJECT_GENERATOR, ValaProjectGeneratorClass))
#define VALA_IS_PROJECT_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_PROJECT_GENERATOR))
#define VALA_IS_PROJECT_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_PROJECT_GENERATOR))
#define VALA_PROJECT_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_PROJECT_GENERATOR, ValaProjectGeneratorClass))

typedef struct _ValaProjectGenerator ValaProjectGenerator;
typedef struct _ValaProjectGeneratorClass ValaProjectGeneratorClass;
typedef struct _ValaProjectGeneratorPrivate ValaProjectGeneratorPrivate;
typedef struct _ValaParamSpecProjectGenerator ValaParamSpecProjectGenerator;

typedef enum  {
	VALA_PROJECT_TYPE_CONSOLE_APPLICATION,
	VALA_PROJECT_TYPE_GTK_APPLICATION
} ValaProjectType;

typedef enum  {
	VALA_PROJECT_LICENSE_GPL2,
	VALA_PROJECT_LICENSE_GPL3,
	VALA_PROJECT_LICENSE_LGPL2,
	VALA_PROJECT_LICENSE_LGPL3
} ValaProjectLicense;

struct _ValaProjectGenerator {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaProjectGeneratorPrivate * priv;
};

struct _ValaProjectGeneratorClass {
	GTypeClass parent_class;
	void (*finalize) (ValaProjectGenerator *self);
};

struct _ValaParamSpecProjectGenerator {
	GParamSpec parent_instance;
};


GType vala_project_type_get_type (void);
GType vala_project_license_get_type (void);
gboolean vala_project_generator_option_parse_callback (const char* option_name, const char* value, void* data, GError** error);
ValaProjectGenerator* vala_project_generator_construct (GType object_type);
ValaProjectGenerator* vala_project_generator_new (void);
GtkResponseType vala_project_generator_ask_parameters (ValaProjectGenerator* self);
void vala_project_generator_create_project (ValaProjectGenerator* self);
GParamSpec* vala_param_spec_project_generator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
gpointer vala_value_get_project_generator (const GValue* value);
void vala_value_set_project_generator (GValue* value, gpointer v_object);
GType vala_project_generator_get_type (void);
gpointer vala_project_generator_ref (gpointer instance);
void vala_project_generator_unref (gpointer instance);


G_END_DECLS

#endif
