/*
 *  vbfconfignodelist.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vbfconfignodelist.h>
#include <gee/arraylist.h>
#include <gee/readonlylist.h>
#include <gee/collection.h>
#include <gee/iterable.h>
#include <gee/iterator.h>




static glong string_get_length (const char* self);
static char* string_substring (const char* self, glong offset, glong len);
enum  {
	VBF_CONFIG_NODE_LIST_DUMMY_PROPERTY
};
static char* vbf_config_node_list_real_to_string (VbfConfigNode* base);
static gpointer vbf_config_node_list_parent_class = NULL;
static void vbf_config_node_list_finalize (GObject* obj);



static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


GeeList* vbf_config_node_list_get_values (VbfConfigNodeList* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (VBF_TYPE_CONFIG_NODE, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->values);
}


void vbf_config_node_list_add_value (VbfConfigNodeList* self, VbfConfigNode* val) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (val != NULL);
	gee_collection_add ((GeeCollection*) self->values, val);
}


void vbf_config_node_list_replace_config_node (VbfConfigNodeList* self, VbfConfigNode* source, VbfConfigNode* target) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (source != NULL);
	g_return_if_fail (target != NULL);
	if (gee_collection_contains ((GeeCollection*) self->values, source)) {
		gee_collection_remove ((GeeCollection*) self->values, source);
		if (target != NULL) {
			gee_collection_add ((GeeCollection*) self->values, target);
		}
	}
}


static char* vbf_config_node_list_real_to_string (VbfConfigNode* base) {
	VbfConfigNodeList * self;
	char* res;
	self = (VbfConfigNodeList*) base;
	res = g_strdup ("");
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator ((GeeIterable*) self->values);
		while (gee_iterator_next (_item_it)) {
			VbfConfigNode* item;
			item = (VbfConfigNode*) gee_iterator_get (_item_it);
			if (item != NULL) {
				char* _tmp2;
				char* _tmp1;
				char* _tmp0;
				_tmp2 = NULL;
				_tmp1 = NULL;
				_tmp0 = NULL;
				res = (_tmp2 = g_strconcat (res, _tmp1 = g_strdup_printf ("%s, ", _tmp0 = vbf_config_node_to_string (item)), NULL), res = (g_free (res), NULL), _tmp2);
				_tmp1 = (g_free (_tmp1), NULL);
				_tmp0 = (g_free (_tmp0), NULL);
			} else {
				g_critical ("vbfconfignodelist.vala:56: item is null");
			}
			(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
		}
		(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
	}
	if (string_get_length (res) > 2) {
		char* _tmp3;
		_tmp3 = NULL;
		res = (_tmp3 = string_substring (res, (glong) 0, string_get_length (res) - 2), res = (g_free (res), NULL), _tmp3);
	}
	return res;
}


VbfConfigNodeList* vbf_config_node_list_construct (GType object_type) {
	VbfConfigNodeList * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VbfConfigNodeList* vbf_config_node_list_new (void) {
	return vbf_config_node_list_construct (VBF_TYPE_CONFIG_NODE_LIST);
}


static void vbf_config_node_list_class_init (VbfConfigNodeListClass * klass) {
	vbf_config_node_list_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = vbf_config_node_list_finalize;
	VBF_CONFIG_NODE_CLASS (klass)->to_string = vbf_config_node_list_real_to_string;
}


static void vbf_config_node_list_instance_init (VbfConfigNodeList * self) {
	self->values = (GeeList*) gee_array_list_new (VBF_TYPE_CONFIG_NODE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
}


static void vbf_config_node_list_finalize (GObject* obj) {
	VbfConfigNodeList * self;
	self = VBF_CONFIG_NODE_LIST (obj);
	(self->values == NULL) ? NULL : (self->values = (gee_collection_object_unref (self->values), NULL));
	G_OBJECT_CLASS (vbf_config_node_list_parent_class)->finalize (obj);
}


GType vbf_config_node_list_get_type (void) {
	static GType vbf_config_node_list_type_id = 0;
	if (vbf_config_node_list_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfConfigNodeListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_config_node_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfConfigNodeList), 0, (GInstanceInitFunc) vbf_config_node_list_instance_init, NULL };
		vbf_config_node_list_type_id = g_type_register_static (VBF_TYPE_CONFIG_NODE, "VbfConfigNodeList", &g_define_type_info, 0);
	}
	return vbf_config_node_list_type_id;
}




