/*
 *  vbffile.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vbffile.h>
#include <gio/gio.h>
#include <vbftarget.h>




enum  {
	VBF_FILE_DUMMY_PROPERTY
};
static gpointer vbf_file_parent_class = NULL;
static void vbf_file_finalize (GObject* obj);




GType vbf_file_types_get_type (void) {
	static GType vbf_file_types_type_id = 0;
	if (G_UNLIKELY (vbf_file_types_type_id == 0)) {
		static const GEnumValue values[] = {{VBF_FILE_TYPES_UNKNOWN, "VBF_FILE_TYPES_UNKNOWN", "unknown"}, {VBF_FILE_TYPES_DATA, "VBF_FILE_TYPES_DATA", "data"}, {VBF_FILE_TYPES_VALA_SOURCE, "VBF_FILE_TYPES_VALA_SOURCE", "vala-source"}, {0, NULL, NULL}};
		vbf_file_types_type_id = g_enum_register_static ("VbfFileTypes", values);
	}
	return vbf_file_types_type_id;
}


VbfFile* vbf_file_construct (GType object_type, VbfTarget* target, const char* filename) {
	VbfFile * self;
	g_return_val_if_fail (target != NULL, NULL);
	g_return_val_if_fail (filename != NULL, NULL);
	self = (VbfFile*) vbf_file_construct_with_type (object_type, target, filename, VBF_FILE_TYPES_UNKNOWN);
	return self;
}


VbfFile* vbf_file_new (VbfTarget* target, const char* filename) {
	return vbf_file_construct (VBF_TYPE_FILE, target, filename);
}


VbfFile* vbf_file_construct_with_type (GType object_type, VbfTarget* target, const char* filename, VbfFileTypes type) {
	GError * inner_error;
	VbfFile * self;
	const char* _tmp0;
	char* file;
	char* _tmp4;
	const char* _tmp3;
	char* _tmp5;
	char* _tmp6;
	char* _tmp7;
	g_return_val_if_fail (target != NULL, NULL);
	g_return_val_if_fail (filename != NULL, NULL);
	inner_error = NULL;
	self = g_object_newv (object_type, 0, NULL);
	_tmp0 = NULL;
	file = (_tmp0 = filename, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (!g_path_is_absolute (file)) {
		GFile* f;
		char* _tmp2;
		GFile* _tmp1;
		f = g_file_new_for_path (file);
		_tmp2 = NULL;
		_tmp1 = NULL;
		file = (_tmp2 = g_file_get_path (_tmp1 = g_file_resolve_relative_path (f, file)), file = (g_free (file), NULL), _tmp2);
		(_tmp1 == NULL) ? NULL : (_tmp1 = (g_object_unref (_tmp1), NULL));
		(f == NULL) ? NULL : (f = (g_object_unref (f), NULL));
	}
	_tmp4 = NULL;
	_tmp3 = NULL;
	self->filename = (_tmp4 = (_tmp3 = file, (_tmp3 == NULL) ? NULL : g_strdup (_tmp3)), self->filename = (g_free (self->filename), NULL), _tmp4);
	_tmp5 = g_filename_to_uri (file, NULL, &inner_error);
	if (inner_error != NULL) {
		file = (g_free (file), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	_tmp6 = NULL;
	self->uri = (_tmp6 = _tmp5, self->uri = (g_free (self->uri), NULL), _tmp6);
	_tmp7 = NULL;
	self->name = (_tmp7 = g_filename_display_basename (file), self->name = (g_free (self->name), NULL), _tmp7);
	self->target = target;
	self->type = type;
	return self;
}


VbfFile* vbf_file_new_with_type (VbfTarget* target, const char* filename, VbfFileTypes type) {
	return vbf_file_construct_with_type (VBF_TYPE_FILE, target, filename, type);
}


static void vbf_file_class_init (VbfFileClass * klass) {
	vbf_file_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = vbf_file_finalize;
}


static void vbf_file_instance_init (VbfFile * self) {
}


static void vbf_file_finalize (GObject* obj) {
	VbfFile * self;
	self = VBF_FILE (obj);
	self->name = (g_free (self->name), NULL);
	self->filename = (g_free (self->filename), NULL);
	self->uri = (g_free (self->uri), NULL);
	G_OBJECT_CLASS (vbf_file_parent_class)->finalize (obj);
}


GType vbf_file_get_type (void) {
	static GType vbf_file_type_id = 0;
	if (vbf_file_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfFileClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_file_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfFile), 0, (GInstanceInitFunc) vbf_file_instance_init, NULL };
		vbf_file_type_id = g_type_register_static (G_TYPE_OBJECT, "VbfFile", &g_define_type_info, 0);
	}
	return vbf_file_type_id;
}




