/*
 *  vbfgroup.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vbfgroup.h>
#include <gee/arraylist.h>
#include <gee/readonlylist.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gee/collection.h>
#include <vbftarget.h>




static char* string_replace (const char* self, const char* old, const char* replacement);
struct _VbfGroupPrivate {
	GeeList* packages;
	GeeList* include_dirs;
	GeeList* built_libraries;
	GeeList* targets;
	GeeList* variables;
	GeeList* subgroups;
};

#define VBF_GROUP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VBF_TYPE_GROUP, VbfGroupPrivate))
enum  {
	VBF_GROUP_DUMMY_PROPERTY
};
static gpointer vbf_group_parent_class = NULL;
static void vbf_group_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



static char* string_replace (const char* self, const char* old, const char* replacement) {
	GError * inner_error;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (old != NULL, NULL);
	g_return_val_if_fail (replacement != NULL, NULL);
	inner_error = NULL;
	{
		char* _tmp0;
		GRegex* _tmp1;
		GRegex* regex;
		char* _tmp2;
		char* _tmp3;
		_tmp0 = NULL;
		_tmp1 = NULL;
		regex = (_tmp1 = g_regex_new (_tmp0 = g_regex_escape_string (old, -1), 0, 0, &inner_error), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
		if (inner_error != NULL) {
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch1_g_regex_error;
			}
			goto __finally1;
		}
		_tmp2 = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &inner_error);
		if (inner_error != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch1_g_regex_error;
			}
			goto __finally1;
		}
		_tmp3 = NULL;
		return (_tmp3 = _tmp2, (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp3);
	}
	goto __finally1;
	__catch1_g_regex_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_assert_not_reached ();
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally1:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return NULL;
	}
}


VbfGroup* vbf_group_construct (GType object_type, VbfProject* project, const char* id) {
	VbfGroup * self;
	char* _tmp1;
	const char* _tmp0;
	char* _tmp2;
	g_return_val_if_fail (project != NULL, NULL);
	g_return_val_if_fail (id != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	self->project = project;
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->id = (_tmp1 = (_tmp0 = id, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->id = (g_free (self->id), NULL), _tmp1);
	_tmp2 = NULL;
	self->name = (_tmp2 = string_replace (id, project->id, ""), self->name = (g_free (self->name), NULL), _tmp2);
	if (g_str_has_prefix (self->name, "/")) {
		char* _tmp6;
		const char* _tmp5;
		gint _tmp4_length1;
		char** _tmp4;
		char** _tmp3;
		_tmp6 = NULL;
		_tmp5 = NULL;
		_tmp4 = NULL;
		_tmp3 = NULL;
		self->name = (_tmp6 = (_tmp5 = (_tmp4 = _tmp3 = g_strsplit (self->name, "/", 0), _tmp4_length1 = _vala_array_length (_tmp3), _tmp4)[1], (_tmp5 == NULL) ? NULL : g_strdup (_tmp5)), self->name = (g_free (self->name), NULL), _tmp6);
		_tmp4 = (_vala_array_free (_tmp4, _tmp4_length1, (GDestroyNotify) g_free), NULL);
	} else {
		char* _tmp10;
		const char* _tmp9;
		gint _tmp8_length1;
		char** _tmp8;
		char** _tmp7;
		_tmp10 = NULL;
		_tmp9 = NULL;
		_tmp8 = NULL;
		_tmp7 = NULL;
		self->name = (_tmp10 = (_tmp9 = (_tmp8 = _tmp7 = g_strsplit (self->name, "/", 0), _tmp8_length1 = _vala_array_length (_tmp7), _tmp8)[0], (_tmp9 == NULL) ? NULL : g_strdup (_tmp9)), self->name = (g_free (self->name), NULL), _tmp10);
		_tmp8 = (_vala_array_free (_tmp8, _tmp8_length1, (GDestroyNotify) g_free), NULL);
	}
	if (_vala_strcmp0 (self->name, "") == 0) {
		char* _tmp11;
		_tmp11 = NULL;
		self->name = (_tmp11 = g_strconcat ("/ - ", project->name, NULL), self->name = (g_free (self->name), NULL), _tmp11);
	}
	return self;
}


VbfGroup* vbf_group_new (VbfProject* project, const char* id) {
	return vbf_group_construct (VBF_TYPE_GROUP, project, id);
}


GeeList* vbf_group_get_targets (VbfGroup* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (VBF_TYPE_TARGET, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->targets);
}


gboolean vbf_group_contains_target (VbfGroup* self, const char* id) {
	VbfTarget* _tmp0;
	gboolean _tmp1;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (id != NULL, FALSE);
	_tmp0 = NULL;
	return (_tmp1 = (_tmp0 = vbf_group_get_target_for_id (self, id)) != NULL, (_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)), _tmp1);
}


VbfTarget* vbf_group_get_target_for_id (VbfGroup* self, const char* id) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (id != NULL, NULL);
	{
		GeeIterator* _target_it;
		_target_it = gee_iterable_iterator ((GeeIterable*) self->priv->targets);
		while (gee_iterator_next (_target_it)) {
			VbfTarget* target;
			target = (VbfTarget*) gee_iterator_get (_target_it);
			if (_vala_strcmp0 (target->id, id) == 0) {
				VbfTarget* _tmp0;
				_tmp0 = NULL;
				return (_tmp0 = target, (_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL)), _tmp0);
			}
			(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
		}
		(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
	}
	return NULL;
}


void vbf_group_add_target (VbfGroup* self, VbfTarget* target) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (target != NULL);
	gee_collection_add ((GeeCollection*) self->priv->targets, target);
}


GeeList* vbf_group_get_packages (VbfGroup* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (VBF_TYPE_PACKAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->packages);
}


void vbf_group_add_package (VbfGroup* self, VbfPackage* package) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (package != NULL);
	gee_collection_add ((GeeCollection*) self->priv->packages, package);
}


GeeList* vbf_group_get_include_dirs (VbfGroup* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, self->priv->include_dirs);
}


void vbf_group_add_include_dir (VbfGroup* self, const char* dir) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (dir != NULL);
	gee_collection_add ((GeeCollection*) self->priv->include_dirs, dir);
}


GeeList* vbf_group_get_built_libraries (VbfGroup* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, self->priv->built_libraries);
}


void vbf_group_add_built_library (VbfGroup* self, const char* dir) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (dir != NULL);
	gee_collection_add ((GeeCollection*) self->priv->built_libraries, dir);
}


GeeList* vbf_group_get_subgroups (VbfGroup* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (VBF_TYPE_GROUP, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->subgroups);
}


GeeList* vbf_group_get_variables (VbfGroup* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (VBF_TYPE_VARIABLE, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->variables);
}


void vbf_group_add_variable (VbfGroup* self, VbfVariable* variable) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (variable != NULL);
	gee_collection_add ((GeeCollection*) self->priv->variables, variable);
}


static void vbf_group_class_init (VbfGroupClass * klass) {
	vbf_group_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VbfGroupPrivate));
	G_OBJECT_CLASS (klass)->finalize = vbf_group_finalize;
}


static void vbf_group_instance_init (VbfGroup * self) {
	self->priv = VBF_GROUP_GET_PRIVATE (self);
	self->project = NULL;
	self->priv->packages = (GeeList*) gee_array_list_new (VBF_TYPE_PACKAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->include_dirs = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
	self->priv->built_libraries = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
	self->priv->targets = (GeeList*) gee_array_list_new (VBF_TYPE_TARGET, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->variables = (GeeList*) gee_array_list_new (VBF_TYPE_VARIABLE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->subgroups = (GeeList*) gee_array_list_new (VBF_TYPE_GROUP, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
}


static void vbf_group_finalize (GObject* obj) {
	VbfGroup * self;
	self = VBF_GROUP (obj);
	self->name = (g_free (self->name), NULL);
	self->id = (g_free (self->id), NULL);
	(self->priv->packages == NULL) ? NULL : (self->priv->packages = (gee_collection_object_unref (self->priv->packages), NULL));
	(self->priv->include_dirs == NULL) ? NULL : (self->priv->include_dirs = (gee_collection_object_unref (self->priv->include_dirs), NULL));
	(self->priv->built_libraries == NULL) ? NULL : (self->priv->built_libraries = (gee_collection_object_unref (self->priv->built_libraries), NULL));
	(self->priv->targets == NULL) ? NULL : (self->priv->targets = (gee_collection_object_unref (self->priv->targets), NULL));
	(self->priv->variables == NULL) ? NULL : (self->priv->variables = (gee_collection_object_unref (self->priv->variables), NULL));
	(self->priv->subgroups == NULL) ? NULL : (self->priv->subgroups = (gee_collection_object_unref (self->priv->subgroups), NULL));
	G_OBJECT_CLASS (vbf_group_parent_class)->finalize (obj);
}


GType vbf_group_get_type (void) {
	static GType vbf_group_type_id = 0;
	if (vbf_group_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfGroupClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_group_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfGroup), 0, (GInstanceInitFunc) vbf_group_instance_init, NULL };
		vbf_group_type_id = g_type_register_static (G_TYPE_OBJECT, "VbfGroup", &g_define_type_info, 0);
	}
	return vbf_group_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




