/*
 *  vbfgroup.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#ifndef __VBFGROUP_H__
#define __VBFGROUP_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee/list.h>
#include <vbfproject.h>
#include <vbfpackage.h>
#include <vbfvariable.h>

G_BEGIN_DECLS


#define VBF_TYPE_GROUP (vbf_group_get_type ())
#define VBF_GROUP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_GROUP, VbfGroup))
#define VBF_GROUP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_GROUP, VbfGroupClass))
#define VBF_IS_GROUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_GROUP))
#define VBF_IS_GROUP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_GROUP))
#define VBF_GROUP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_GROUP, VbfGroupClass))

typedef struct _VbfGroupPrivate VbfGroupPrivate;

struct _VbfGroup {
	GObject parent_instance;
	VbfGroupPrivate * priv;
	char* name;
	char* id;
	VbfProject* project;
};

struct _VbfGroupClass {
	GObjectClass parent_class;
};


VbfGroup* vbf_group_construct (GType object_type, VbfProject* project, const char* id);
VbfGroup* vbf_group_new (VbfProject* project, const char* id);
GeeList* vbf_group_get_targets (VbfGroup* self);
gboolean vbf_group_contains_target (VbfGroup* self, const char* id);
VbfTarget* vbf_group_get_target_for_id (VbfGroup* self, const char* id);
void vbf_group_add_target (VbfGroup* self, VbfTarget* target);
GeeList* vbf_group_get_packages (VbfGroup* self);
void vbf_group_add_package (VbfGroup* self, VbfPackage* package);
GeeList* vbf_group_get_include_dirs (VbfGroup* self);
void vbf_group_add_include_dir (VbfGroup* self, const char* dir);
GeeList* vbf_group_get_built_libraries (VbfGroup* self);
void vbf_group_add_built_library (VbfGroup* self, const char* dir);
GeeList* vbf_group_get_subgroups (VbfGroup* self);
GeeList* vbf_group_get_variables (VbfGroup* self);
void vbf_group_add_variable (VbfGroup* self, VbfVariable* variable);
GType vbf_group_get_type (void);


G_END_DECLS

#endif
