/*
 *  vbfmodule.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vbfmodule.h>
#include <gee/arraylist.h>
#include <gee/readonlylist.h>
#include <gee/collection.h>




struct _VbfModulePrivate {
	GeeList* packages;
};

#define VBF_MODULE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VBF_TYPE_MODULE, VbfModulePrivate))
enum  {
	VBF_MODULE_DUMMY_PROPERTY
};
static gpointer vbf_module_parent_class = NULL;
static void vbf_module_finalize (GObject* obj);



VbfModule* vbf_module_construct (GType object_type, VbfProject* project, const char* id) {
	VbfModule * self;
	char* _tmp1;
	const char* _tmp0;
	char* _tmp3;
	const char* _tmp2;
	g_return_val_if_fail (project != NULL, NULL);
	g_return_val_if_fail (id != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->id = (_tmp1 = (_tmp0 = id, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->id = (g_free (self->id), NULL), _tmp1);
	_tmp3 = NULL;
	_tmp2 = NULL;
	self->name = (_tmp3 = (_tmp2 = id, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), self->name = (g_free (self->name), NULL), _tmp3);
	self->project = project;
	return self;
}


VbfModule* vbf_module_new (VbfProject* project, const char* id) {
	return vbf_module_construct (VBF_TYPE_MODULE, project, id);
}


GeeList* vbf_module_get_packages (VbfModule* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (VBF_TYPE_PACKAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->packages);
}


void vbf_module_add_package (VbfModule* self, VbfPackage* package) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (package != NULL);
	gee_collection_add ((GeeCollection*) self->priv->packages, package);
}


static void vbf_module_class_init (VbfModuleClass * klass) {
	vbf_module_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VbfModulePrivate));
	G_OBJECT_CLASS (klass)->finalize = vbf_module_finalize;
}


static void vbf_module_instance_init (VbfModule * self) {
	self->priv = VBF_MODULE_GET_PRIVATE (self);
	self->priv->packages = (GeeList*) gee_array_list_new (VBF_TYPE_PACKAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
}


static void vbf_module_finalize (GObject* obj) {
	VbfModule * self;
	self = VBF_MODULE (obj);
	self->id = (g_free (self->id), NULL);
	self->name = (g_free (self->name), NULL);
	(self->priv->packages == NULL) ? NULL : (self->priv->packages = (gee_collection_object_unref (self->priv->packages), NULL));
	G_OBJECT_CLASS (vbf_module_parent_class)->finalize (obj);
}


GType vbf_module_get_type (void) {
	static GType vbf_module_type_id = 0;
	if (vbf_module_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfModuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_module_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfModule), 0, (GInstanceInitFunc) vbf_module_instance_init, NULL };
		vbf_module_type_id = g_type_register_static (G_TYPE_OBJECT, "VbfModule", &g_define_type_info, 0);
	}
	return vbf_module_type_id;
}




