/*
 *  vbfsource.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vbfsource.h>
#include <vbftarget.h>




enum  {
	VBF_SOURCE_DUMMY_PROPERTY
};
static gpointer vbf_source_parent_class = NULL;



VbfSource* vbf_source_construct (GType object_type, VbfTarget* target, const char* filename) {
	VbfSource * self;
	g_return_val_if_fail (target != NULL, NULL);
	g_return_val_if_fail (filename != NULL, NULL);
	self = (VbfSource*) vbf_source_construct_with_type (object_type, target, filename, VBF_FILE_TYPES_UNKNOWN);
	return self;
}


VbfSource* vbf_source_new (VbfTarget* target, const char* filename) {
	return vbf_source_construct (VBF_TYPE_SOURCE, target, filename);
}


VbfSource* vbf_source_construct_with_type (GType object_type, VbfTarget* target, const char* filename, VbfFileTypes type) {
	VbfSource * self;
	g_return_val_if_fail (target != NULL, NULL);
	g_return_val_if_fail (filename != NULL, NULL);
	self = (VbfSource*) vbf_file_construct_with_type (object_type, target, filename, type);
	return self;
}


VbfSource* vbf_source_new_with_type (VbfTarget* target, const char* filename, VbfFileTypes type) {
	return vbf_source_construct_with_type (VBF_TYPE_SOURCE, target, filename, type);
}


static void vbf_source_class_init (VbfSourceClass * klass) {
	vbf_source_parent_class = g_type_class_peek_parent (klass);
}


static void vbf_source_instance_init (VbfSource * self) {
}


GType vbf_source_get_type (void) {
	static GType vbf_source_type_id = 0;
	if (vbf_source_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfSourceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_source_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfSource), 0, (GInstanceInitFunc) vbf_source_instance_init, NULL };
		vbf_source_type_id = g_type_register_static (VBF_TYPE_FILE, "VbfSource", &g_define_type_info, 0);
	}
	return vbf_source_type_id;
}




