/*
 *  vbftarget.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#ifndef __VBFTARGET_H__
#define __VBFTARGET_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee/list.h>
#include <vbffile.h>
#include <vbfproject.h>
#include <vbfpackage.h>

G_BEGIN_DECLS


#define VBF_TYPE_TARGET_TYPES (vbf_target_types_get_type ())

#define VBF_TYPE_TARGET (vbf_target_get_type ())
#define VBF_TARGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_TARGET, VbfTarget))
#define VBF_TARGET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_TARGET, VbfTargetClass))
#define VBF_IS_TARGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_TARGET))
#define VBF_IS_TARGET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_TARGET))
#define VBF_TARGET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_TARGET, VbfTargetClass))

typedef struct _VbfTargetPrivate VbfTargetPrivate;

typedef enum  {
	VBF_TARGET_TYPES_PROGRAM,
	VBF_TARGET_TYPES_LIBRARY,
	VBF_TARGET_TYPES_DATA,
	VBF_TARGET_TYPES_BUILT_SOURCES
} VbfTargetTypes;

struct _VbfTarget {
	GObject parent_instance;
	VbfTargetPrivate * priv;
	char* id;
	char* name;
	VbfTargetTypes type;
	gboolean no_install;
	VbfGroup* group;
};

struct _VbfTargetClass {
	GObjectClass parent_class;
};


GType vbf_target_types_get_type (void);
VbfTarget* vbf_target_construct (GType object_type, VbfGroup* group, VbfTargetTypes type, const char* id);
VbfTarget* vbf_target_new (VbfGroup* group, VbfTargetTypes type, const char* id);
GeeList* vbf_target_get_sources (VbfTarget* self);
gboolean vbf_target_has_sources_of_type (VbfTarget* self, VbfFileTypes type);
void vbf_target_add_source (VbfTarget* self, VbfSource* source);
gboolean vbf_target_has_file_of_type (VbfTarget* self, VbfFileTypes type);
GeeList* vbf_target_get_files (VbfTarget* self);
void vbf_target_add_file (VbfTarget* self, VbfFile* file);
GeeList* vbf_target_get_packages (VbfTarget* self);
void vbf_target_add_package (VbfTarget* self, VbfPackage* package);
GeeList* vbf_target_get_include_dirs (VbfTarget* self);
void vbf_target_add_include_dir (VbfTarget* self, const char* dir);
GeeList* vbf_target_get_built_libraries (VbfTarget* self);
void vbf_target_add_built_library (VbfTarget* self, const char* dir);
GType vbf_target_get_type (void);


G_END_DECLS

#endif
