/*
 *  vbfvariable.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <vbfvariable.h>
#include <gee/arraylist.h>
#include <gee/readonlylist.h>
#include <gee/collection.h>




struct _VbfVariablePrivate {
	GeeList* childs;
};

#define VBF_VARIABLE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VBF_TYPE_VARIABLE, VbfVariablePrivate))
enum  {
	VBF_VARIABLE_DUMMY_PROPERTY
};
static char* vbf_variable_real_to_string (VbfConfigNode* base);
static gpointer vbf_variable_parent_class = NULL;
static void vbf_variable_finalize (GObject* obj);



VbfVariable* vbf_variable_construct (GType object_type, const char* name, VbfConfigNode* parent) {
	VbfVariable * self;
	char* _tmp1;
	const char* _tmp0;
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (parent != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->name = (_tmp1 = (_tmp0 = name, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), self->name = (g_free (self->name), NULL), _tmp1);
	return self;
}


VbfVariable* vbf_variable_new (const char* name, VbfConfigNode* parent) {
	return vbf_variable_construct (VBF_TYPE_VARIABLE, name, parent);
}


GeeList* vbf_variable_get_childs (VbfVariable* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GeeList*) gee_read_only_list_new (VBF_TYPE_VARIABLE, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->childs);
}


void vbf_variable_add_child (VbfVariable* self, VbfVariable* variable) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (variable != NULL);
	gee_collection_add ((GeeCollection*) self->priv->childs, variable);
}


static char* vbf_variable_real_to_string (VbfConfigNode* base) {
	VbfVariable * self;
	char* res;
	self = (VbfVariable*) base;
	res = NULL;
	if (VBF_IS_VARIABLE (self->data)) {
		char* _tmp0;
		_tmp0 = NULL;
		res = (_tmp0 = g_strdup_printf ("$(%s)=", self->name), res = (g_free (res), NULL), _tmp0);
	} else {
		char* _tmp1;
		_tmp1 = NULL;
		res = (_tmp1 = g_strdup_printf ("%s=", self->name), res = (g_free (res), NULL), _tmp1);
	}
	if (self->data == NULL) {
		char* _tmp2;
		_tmp2 = NULL;
		res = (_tmp2 = g_strconcat (res, "(null)", NULL), res = (g_free (res), NULL), _tmp2);
	} else {
		char* _tmp4;
		char* _tmp3;
		_tmp4 = NULL;
		_tmp3 = NULL;
		res = (_tmp4 = g_strconcat (res, _tmp3 = vbf_config_node_to_string (self->data), NULL), res = (g_free (res), NULL), _tmp4);
		_tmp3 = (g_free (_tmp3), NULL);
	}
	return res;
}


VbfConfigNode* vbf_variable_get_value (VbfVariable* self) {
	g_return_val_if_fail (self != NULL, NULL);
	if (VBF_IS_VARIABLE (self->data)) {
		return vbf_variable_get_value (VBF_VARIABLE (self->data));
	} else {
		VbfConfigNode* _tmp1;
		_tmp1 = NULL;
		return (_tmp1 = self->data, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	}
}


static void vbf_variable_class_init (VbfVariableClass * klass) {
	vbf_variable_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VbfVariablePrivate));
	G_OBJECT_CLASS (klass)->finalize = vbf_variable_finalize;
	VBF_CONFIG_NODE_CLASS (klass)->to_string = vbf_variable_real_to_string;
}


static void vbf_variable_instance_init (VbfVariable * self) {
	self->priv = VBF_VARIABLE_GET_PRIVATE (self);
	self->data = NULL;
	self->priv->childs = (GeeList*) gee_array_list_new (VBF_TYPE_VARIABLE, NULL, NULL, g_direct_equal);
}


static void vbf_variable_finalize (GObject* obj) {
	VbfVariable * self;
	self = VBF_VARIABLE (obj);
	self->name = (g_free (self->name), NULL);
	(self->data == NULL) ? NULL : (self->data = (g_object_unref (self->data), NULL));
	(self->priv->childs == NULL) ? NULL : (self->priv->childs = (gee_collection_object_unref (self->priv->childs), NULL));
	G_OBJECT_CLASS (vbf_variable_parent_class)->finalize (obj);
}


GType vbf_variable_get_type (void) {
	static GType vbf_variable_type_id = 0;
	if (vbf_variable_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfVariableClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_variable_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfVariable), 0, (GInstanceInitFunc) vbf_variable_instance_init, NULL };
		vbf_variable_type_id = g_type_register_static (VBF_TYPE_CONFIG_NODE, "VbfVariable", &g_define_type_info, 0);
	}
	return vbf_variable_type_id;
}




