/*
 *  vtgplugin.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgplugin.h"
#include <gee/arraylist.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gee/collection.h>
#include <stdlib.h>
#include <string.h>
#include <vscsymbolcompletion.h>
#include <vscparsermanager.h>
#include <gtksourceview/gtksourceview.h>
#include <vbfproject.h>
#include <vbfmodule.h>
#include <vbfpackage.h>
#include <vbfgroup.h>
#include <vbftarget.h>
#include <vbfsource.h>
#include <vbffile.h>
#include <gedit/gedit-tab.h>
#include <glib/gi18n-lib.h>
#include <config.h>
#include "vtgplugininstance.h"
#include "vtgprojectmanagerui.h"
#include "vtgprojectview.h"
#include "vtgutils.h"


#define VTG_PLUGIN_TYPE_DEACTIVATE_MODULE_OPTIONS (vtg_plugin_deactivate_module_options_get_type ())

typedef enum  {
	VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL,
	VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET,
	VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL
} VtgPluginDeactivateModuleOptions;



struct _VtgPluginPrivate {
	GeeList* _instances;
	GeeList* _projects;
	VtgConfiguration* _config;
	VtgProjectDescriptor* _default_project;
};

#define VTG_PLUGIN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_PLUGIN, VtgPluginPrivate))
enum  {
	VTG_PLUGIN_DUMMY_PROPERTY,
	VTG_PLUGIN_PROJECTS,
	VTG_PLUGIN_CONFIG
};
GType vtg_plugin_deactivate_module_options_get_type (void);
static VtgPluginInstance* vtg_plugin_get_plugin_instance_for_window (VtgPlugin* self, GeditWindow* window);
static void vtg_plugin_real_activate (GeditPlugin* base, GeditWindow* window);
static void vtg_plugin_real_deactivate (GeditPlugin* base, GeditWindow* window);
static gboolean vtg_plugin_real_is_configurable (GeditPlugin* base);
static GtkWidget* vtg_plugin_real_create_configure_dialog (GeditPlugin* base);
static void vtg_plugin_real_update_ui (GeditPlugin* base, GeditWindow* window);
static void vtg_plugin_on_configuration_property_changed (VtgPlugin* self, GObject* sender, GParamSpec* param);
static void vtg_plugin_deactivate_modules (VtgPlugin* self, VtgPluginDeactivateModuleOptions options);
static void vtg_plugin_activate_modules (VtgPlugin* self, VtgPluginDeactivateModuleOptions options);
static void _vtg_plugin_on_configuration_property_changed_g_object_notify (VtgConfiguration* _sender, GParamSpec* pspec, gpointer self);
static GObject * vtg_plugin_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_plugin_parent_class = NULL;
static void vtg_plugin_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);




GType vtg_plugin_deactivate_module_options_get_type (void) {
	static GType vtg_plugin_deactivate_module_options_type_id = 0;
	if (G_UNLIKELY (vtg_plugin_deactivate_module_options_type_id == 0)) {
		static const GEnumValue values[] = {{VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL, "VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL", "all"}, {VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET, "VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET", "bracket"}, {VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL, "VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL", "symbol"}, {0, NULL, NULL}};
		vtg_plugin_deactivate_module_options_type_id = g_enum_register_static ("VtgPluginDeactivateModuleOptions", values);
	}
	return vtg_plugin_deactivate_module_options_type_id;
}


static VtgPluginInstance* vtg_plugin_get_plugin_instance_for_window (VtgPlugin* self, GeditWindow* window) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (window != NULL, NULL);
	{
		GeeIterator* _instance_it;
		_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
		while (gee_iterator_next (_instance_it)) {
			VtgPluginInstance* instance;
			instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
			if (vtg_plugin_instance_get_window (instance) == window) {
				VtgPluginInstance* _tmp0;
				_tmp0 = NULL;
				return (_tmp0 = instance, (_instance_it == NULL) ? NULL : (_instance_it = (gee_collection_object_unref (_instance_it), NULL)), _tmp0);
			}
			(instance == NULL) ? NULL : (instance = (g_object_unref (instance), NULL));
		}
		(_instance_it == NULL) ? NULL : (_instance_it = (gee_collection_object_unref (_instance_it), NULL));
	}
	return NULL;
}


static void vtg_plugin_real_activate (GeditPlugin* base, GeditWindow* window) {
	VtgPlugin * self;
	VtgPluginInstance* _tmp0;
	gboolean _tmp1;
	self = (VtgPlugin*) base;
	g_return_if_fail (window != NULL);
	_tmp0 = NULL;
	if ((_tmp1 = (_tmp0 = vtg_plugin_get_plugin_instance_for_window (self, window)) == NULL, (_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)), _tmp1)) {
		VtgPluginInstance* instance;
		instance = vtg_plugin_instance_new (self, window);
		gee_collection_add ((GeeCollection*) self->priv->_instances, instance);
		(instance == NULL) ? NULL : (instance = (g_object_unref (instance), NULL));
	}
}


static void vtg_plugin_real_deactivate (GeditPlugin* base, GeditWindow* window) {
	VtgPlugin * self;
	self = (VtgPlugin*) base;
	g_return_if_fail (window != NULL);
	vtg_plugin_deactivate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL);
	gee_collection_clear ((GeeCollection*) self->priv->_instances);
}


static gboolean vtg_plugin_real_is_configurable (GeditPlugin* base) {
	VtgPlugin * self;
	self = (VtgPlugin*) base;
	return TRUE;
}


static GtkWidget* vtg_plugin_real_create_configure_dialog (GeditPlugin* base) {
	VtgPlugin * self;
	self = (VtgPlugin*) base;
	return vtg_configuration_get_configuration_dialog (self->priv->_config);
}


static void vtg_plugin_real_update_ui (GeditPlugin* base, GeditWindow* window) {
	VtgPlugin * self;
	GeditDocument* _tmp0;
	GeditDocument* doc;
	VtgPluginInstance* instance;
	self = (VtgPlugin*) base;
	g_return_if_fail (window != NULL);
	_tmp0 = NULL;
	doc = (_tmp0 = gedit_window_get_active_document (window), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	instance = vtg_plugin_get_plugin_instance_for_window (self, window);
	if (doc != NULL) {
		VtgProjectDescriptor* prj;
		gboolean _tmp1;
		prj = vtg_plugin_project_descriptor_find_from_document (self, doc);
		_tmp1 = FALSE;
		if (prj->project == NULL) {
			gboolean _tmp2;
			_tmp2 = FALSE;
			if (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc) == NULL) {
				_tmp2 = TRUE;
			} else {
				_tmp2 = _vala_strcmp0 (gtk_source_language_get_id (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc)), "vala") != 0;
			}
			_tmp1 = _tmp2;
		} else {
			_tmp1 = FALSE;
		}
		if (!_tmp1) {
			vtg_project_view_set_current_project (vtg_project_manager_ui_get_project_view (vtg_plugin_instance_get_project_manager_ui (instance)), prj->project);
		}
		(prj == NULL) ? NULL : (prj = (g_object_unref (prj), NULL));
	}
	(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
	(instance == NULL) ? NULL : (instance = (g_object_unref (instance), NULL));
}


static void vtg_plugin_on_configuration_property_changed (VtgPlugin* self, GObject* sender, GParamSpec* param) {
	const char* _tmp0;
	char* name;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (param != NULL);
	_tmp0 = NULL;
	name = (_tmp0 = g_param_spec_get_name (param), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (_vala_strcmp0 (name, "bracket-enabled") == 0) {
		if (vtg_configuration_get_bracket_enabled (self->priv->_config)) {
			vtg_plugin_activate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET);
		} else {
			vtg_plugin_deactivate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET);
		}
	} else {
		if (_vala_strcmp0 (name, "symbol-enabled") == 0) {
			if (vtg_configuration_get_bracket_enabled (self->priv->_config)) {
				vtg_plugin_activate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL);
			} else {
				vtg_plugin_deactivate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL);
			}
		}
	}
	name = (g_free (name), NULL);
}


static void vtg_plugin_deactivate_modules (VtgPlugin* self, VtgPluginDeactivateModuleOptions options) {
	gboolean _tmp0;
	gboolean _tmp1;
	g_return_if_fail (self != NULL);
	_tmp0 = FALSE;
	if (options == VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = options == VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL;
	}
	if (_tmp0) {
		{
			GeeIterator* _instance_it;
			_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
			while (gee_iterator_next (_instance_it)) {
				VtgPluginInstance* instance;
				instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
				vtg_plugin_instance_deactivate_symbols (instance);
				(instance == NULL) ? NULL : (instance = (g_object_unref (instance), NULL));
			}
			(_instance_it == NULL) ? NULL : (_instance_it = (gee_collection_object_unref (_instance_it), NULL));
		}
	}
	_tmp1 = FALSE;
	if (options == VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL) {
		_tmp1 = TRUE;
	} else {
		_tmp1 = options == VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET;
	}
	if (_tmp1) {
		{
			GeeIterator* _instance_it;
			_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
			while (gee_iterator_next (_instance_it)) {
				VtgPluginInstance* instance;
				instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
				vtg_plugin_instance_deactivate_brackets (instance);
				(instance == NULL) ? NULL : (instance = (g_object_unref (instance), NULL));
			}
			(_instance_it == NULL) ? NULL : (_instance_it = (gee_collection_object_unref (_instance_it), NULL));
		}
	}
}


static void vtg_plugin_activate_modules (VtgPlugin* self, VtgPluginDeactivateModuleOptions options) {
	g_return_if_fail (self != NULL);
	{
		GeeIterator* _instance_it;
		_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
		while (gee_iterator_next (_instance_it)) {
			VtgPluginInstance* instance;
			instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
			vtg_plugin_instance_initialize_views (instance);
			(instance == NULL) ? NULL : (instance = (g_object_unref (instance), NULL));
		}
		(_instance_it == NULL) ? NULL : (_instance_it = (gee_collection_object_unref (_instance_it), NULL));
	}
}


VtgProjectDescriptor* vtg_plugin_project_descriptor_find_from_document (VtgPlugin* self, GeditDocument* document) {
	const char* _tmp0;
	char* file;
	VtgProjectDescriptor* _tmp6;
	VtgProjectDescriptor* _tmp7;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (document != NULL, NULL);
	_tmp0 = NULL;
	file = (_tmp0 = gedit_document_get_uri (document), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
	if (file == NULL) {
		char* _tmp2;
		const char* _tmp1;
		_tmp2 = NULL;
		_tmp1 = NULL;
		file = (_tmp2 = (_tmp1 = gedit_document_get_short_name_for_display (document), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), file = (g_free (file), NULL), _tmp2);
	}
	{
		GeeIterator* _project_it;
		_project_it = gee_iterable_iterator ((GeeIterable*) self->priv->_projects);
		while (gee_iterator_next (_project_it)) {
			VtgProjectDescriptor* project;
			project = (VtgProjectDescriptor*) gee_iterator_get (_project_it);
			if (vtg_project_manager_contains_file (project->project, file)) {
				VtgProjectDescriptor* _tmp3;
				_tmp3 = NULL;
				return (_tmp3 = project, (_project_it == NULL) ? NULL : (_project_it = (gee_collection_object_unref (_project_it), NULL)), file = (g_free (file), NULL), _tmp3);
			}
			(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
		}
		(_project_it == NULL) ? NULL : (_project_it = (gee_collection_object_unref (_project_it), NULL));
	}
	/*return default_project anyway*/
	if (self->priv->_default_project == NULL) {
		VtgProjectDescriptor* _tmp4;
		VscSymbolCompletion* _tmp5;
		_tmp4 = NULL;
		self->priv->_default_project = (_tmp4 = vtg_project_descriptor_new (), (self->priv->_default_project == NULL) ? NULL : (self->priv->_default_project = (g_object_unref (self->priv->_default_project), NULL)), _tmp4);
		_tmp5 = NULL;
		self->priv->_default_project->completion = (_tmp5 = vsc_symbol_completion_new (), (self->priv->_default_project->completion == NULL) ? NULL : (self->priv->_default_project->completion = (g_object_unref (self->priv->_default_project->completion), NULL)), _tmp5);
		vsc_parser_manager_resume_parsing (vsc_symbol_completion_get_parser (self->priv->_default_project->completion));
	}
	_tmp6 = NULL;
	_tmp7 = NULL;
	return (_tmp7 = (_tmp6 = self->priv->_default_project, (_tmp6 == NULL) ? NULL : g_object_ref (_tmp6)), file = (g_free (file), NULL), _tmp7);
}


void vtg_plugin_on_project_closed (VtgPlugin* self, VtgProjectManagerUi* sender, VtgProjectManager* project) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (project != NULL);
	g_return_if_fail (project != self->priv->_default_project->project);
	{
		GeeIterator* _instance_it;
		_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
		while (gee_iterator_next (_instance_it)) {
			VtgPluginInstance* instance;
			instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
			{
				GList* doc_collection;
				GList* doc_it;
				doc_collection = gedit_window_get_documents (vtg_plugin_instance_get_window (instance));
				for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
					GeditDocument* _tmp1;
					GeditDocument* doc;
					_tmp1 = NULL;
					doc = (_tmp1 = (GeditDocument*) doc_it->data, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
					{
						if (vtg_project_manager_contains_file (project, gedit_document_get_uri (doc))) {
							GeditTab* _tmp0;
							GeditTab* tab;
							/*close tab*/
							_tmp0 = NULL;
							tab = (_tmp0 = gedit_tab_get_from_document (doc), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
							gedit_window_close_tab (vtg_plugin_instance_get_window (instance), tab);
							(tab == NULL) ? NULL : (tab = (g_object_unref (tab), NULL));
						}
						(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
					}
				}
			}
			if (vtg_plugin_instance_get_project_manager_ui (instance) != sender) {
				vtg_project_view_remove_project (vtg_project_manager_ui_get_project_view (vtg_plugin_instance_get_project_manager_ui (instance)), vtg_project_manager_get_project (project));
			}
			(instance == NULL) ? NULL : (instance = (g_object_unref (instance), NULL));
		}
		(_instance_it == NULL) ? NULL : (_instance_it = (gee_collection_object_unref (_instance_it), NULL));
	}
	{
		GeeIterator* _descriptor_it;
		_descriptor_it = gee_iterable_iterator ((GeeIterable*) self->priv->_projects);
		while (gee_iterator_next (_descriptor_it)) {
			VtgProjectDescriptor* descriptor;
			descriptor = (VtgProjectDescriptor*) gee_iterator_get (_descriptor_it);
			if (descriptor->project == project) {
				gee_collection_remove ((GeeCollection*) self->priv->_projects, descriptor);
				(descriptor == NULL) ? NULL : (descriptor = (g_object_unref (descriptor), NULL));
				break;
			}
			(descriptor == NULL) ? NULL : (descriptor = (g_object_unref (descriptor), NULL));
		}
		(_descriptor_it == NULL) ? NULL : (_descriptor_it = (gee_collection_object_unref (_descriptor_it), NULL));
	}
}


gboolean vtg_plugin_project_need_save (VtgPlugin* self, VtgProjectManager* project) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (project != NULL, FALSE);
	{
		GeeIterator* _instance_it;
		_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
		while (gee_iterator_next (_instance_it)) {
			VtgPluginInstance* instance;
			instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
			{
				GList* doc_collection;
				GList* doc_it;
				doc_collection = gedit_window_get_unsaved_documents (vtg_plugin_instance_get_window (instance));
				for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
					GeditDocument* _tmp1;
					GeditDocument* doc;
					_tmp1 = NULL;
					doc = (_tmp1 = (GeditDocument*) doc_it->data, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
					{
						if (vtg_project_manager_contains_file (project, gedit_document_get_uri (doc))) {
							gboolean _tmp0;
							return (_tmp0 = TRUE, (doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL)), (instance == NULL) ? NULL : (instance = (g_object_unref (instance), NULL)), (_instance_it == NULL) ? NULL : (_instance_it = (gee_collection_object_unref (_instance_it), NULL)), _tmp0);
						}
						(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
					}
				}
			}
			(instance == NULL) ? NULL : (instance = (g_object_unref (instance), NULL));
		}
		(_instance_it == NULL) ? NULL : (_instance_it = (gee_collection_object_unref (_instance_it), NULL));
	}
	return FALSE;
}


void vtg_plugin_project_save_all (VtgPlugin* self, VtgProjectManager* project) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	{
		GeeIterator* _instance_it;
		_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
		while (gee_iterator_next (_instance_it)) {
			VtgPluginInstance* instance;
			instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
			{
				GList* doc_collection;
				GList* doc_it;
				doc_collection = gedit_window_get_unsaved_documents (vtg_plugin_instance_get_window (instance));
				for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
					GeditDocument* _tmp2;
					GeditDocument* doc;
					_tmp2 = NULL;
					doc = (_tmp2 = (GeditDocument*) doc_it->data, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
					{
						const char* _tmp0;
						char* uri;
						gboolean _tmp1;
						_tmp0 = NULL;
						uri = (_tmp0 = gedit_document_get_uri (doc), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
						_tmp1 = FALSE;
						if (!vtg_string_utils_is_null_or_empty (uri)) {
							_tmp1 = vtg_project_manager_contains_file (project, uri);
						} else {
							_tmp1 = FALSE;
						}
						if (_tmp1) {
							gedit_document_save (doc, GEDIT_DOCUMENT_SAVE_IGNORE_MTIME);
						}
						(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
						uri = (g_free (uri), NULL);
					}
				}
			}
			(instance == NULL) ? NULL : (instance = (g_object_unref (instance), NULL));
		}
		(_instance_it == NULL) ? NULL : (_instance_it = (gee_collection_object_unref (_instance_it), NULL));
	}
}


void vtg_plugin_on_project_loaded (VtgPlugin* self, VtgProjectManagerUi* sender, VtgProjectManager* project_manager) {
	GError * inner_error;
	VtgProjectDescriptor* prj;
	VscSymbolCompletion* completion;
	VbfProject* _tmp0;
	VbfProject* project;
	VscSymbolCompletion* _tmp32;
	VscSymbolCompletion* _tmp31;
	VtgProjectManager* _tmp34;
	VtgProjectManager* _tmp33;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (project_manager != NULL);
	inner_error = NULL;
	prj = vtg_project_descriptor_new ();
	completion = vsc_symbol_completion_new ();
	_tmp0 = NULL;
	project = (_tmp0 = vtg_project_manager_get_project (project_manager), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	/* setup referenced packages */
	{
		GeeList* _tmp1;
		GeeIterator* _tmp2;
		GeeIterator* _module_it;
		/* setup referenced packages */
		_tmp1 = NULL;
		_tmp2 = NULL;
		_module_it = (_tmp2 = gee_iterable_iterator ((GeeIterable*) (_tmp1 = vbf_project_get_modules (project))), (_tmp1 == NULL) ? NULL : (_tmp1 = (gee_collection_object_unref (_tmp1), NULL)), _tmp2);
		/* setup referenced packages */
		while (gee_iterator_next (_module_it)) {
			VbfModule* module;
			/* setup referenced packages */
			module = (VbfModule*) gee_iterator_get (_module_it);
			{
				GeeList* _tmp3;
				GeeIterator* _tmp4;
				GeeIterator* _package_it;
				_tmp3 = NULL;
				_tmp4 = NULL;
				_package_it = (_tmp4 = gee_iterable_iterator ((GeeIterable*) (_tmp3 = vbf_module_get_packages (module))), (_tmp3 == NULL) ? NULL : (_tmp3 = (gee_collection_object_unref (_tmp3), NULL)), _tmp4);
				while (gee_iterator_next (_package_it)) {
					VbfPackage* package;
					package = (VbfPackage*) gee_iterator_get (_package_it);
					vsc_parser_manager_try_add_package (vsc_symbol_completion_get_parser (completion), package->name, TRUE);
					(package == NULL) ? NULL : (package = (g_object_unref (package), NULL));
				}
				(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
			}
			(module == NULL) ? NULL : (module = (g_object_unref (module), NULL));
		}
		(_module_it == NULL) ? NULL : (_module_it = (gee_collection_object_unref (_module_it), NULL));
	}
	/* first adding a built packages*/
	{
		GeeList* _tmp5;
		GeeIterator* _tmp6;
		GeeIterator* _group_it;
		/* first adding a built packages*/
		_tmp5 = NULL;
		_tmp6 = NULL;
		_group_it = (_tmp6 = gee_iterable_iterator ((GeeIterable*) (_tmp5 = vbf_project_get_groups (project))), (_tmp5 == NULL) ? NULL : (_tmp5 = (gee_collection_object_unref (_tmp5), NULL)), _tmp6);
		/* first adding a built packages*/
		while (gee_iterator_next (_group_it)) {
			VbfGroup* group;
			/* first adding a built packages*/
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp7;
				GeeIterator* _tmp8;
				GeeIterator* _package_it;
				_tmp7 = NULL;
				_tmp8 = NULL;
				_package_it = (_tmp8 = gee_iterable_iterator ((GeeIterable*) (_tmp7 = vbf_group_get_built_libraries (group))), (_tmp7 == NULL) ? NULL : (_tmp7 = (gee_collection_object_unref (_tmp7), NULL)), _tmp8);
				while (gee_iterator_next (_package_it)) {
					char* package;
					package = (char*) gee_iterator_get (_package_it);
					vsc_parser_manager_add_built_package (vsc_symbol_completion_get_parser (completion), package);
					package = (g_free (package), NULL);
				}
				(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
			}
			{
				GeeList* _tmp9;
				GeeIterator* _tmp10;
				GeeIterator* _target_it;
				_tmp9 = NULL;
				_tmp10 = NULL;
				_target_it = (_tmp10 = gee_iterable_iterator ((GeeIterable*) (_tmp9 = vbf_group_get_targets (group))), (_tmp9 == NULL) ? NULL : (_tmp9 = (gee_collection_object_unref (_tmp9), NULL)), _tmp10);
				while (gee_iterator_next (_target_it)) {
					VbfTarget* target;
					target = (VbfTarget*) gee_iterator_get (_target_it);
					{
						GeeList* _tmp11;
						GeeIterator* _tmp12;
						GeeIterator* _package_it;
						_tmp11 = NULL;
						_tmp12 = NULL;
						_package_it = (_tmp12 = gee_iterable_iterator ((GeeIterable*) (_tmp11 = vbf_target_get_built_libraries (target))), (_tmp11 == NULL) ? NULL : (_tmp11 = (gee_collection_object_unref (_tmp11), NULL)), _tmp12);
						while (gee_iterator_next (_package_it)) {
							char* package;
							package = (char*) gee_iterator_get (_package_it);
							vsc_parser_manager_add_built_package (vsc_symbol_completion_get_parser (completion), package);
							package = (g_free (package), NULL);
						}
						(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	/* setup vapidir and local packages */
	{
		GeeList* _tmp13;
		GeeIterator* _tmp14;
		GeeIterator* _group_it;
		/* setup vapidir and local packages */
		_tmp13 = NULL;
		_tmp14 = NULL;
		_group_it = (_tmp14 = gee_iterable_iterator ((GeeIterable*) (_tmp13 = vbf_project_get_groups (project))), (_tmp13 == NULL) ? NULL : (_tmp13 = (gee_collection_object_unref (_tmp13), NULL)), _tmp14);
		/* setup vapidir and local packages */
		while (gee_iterator_next (_group_it)) {
			VbfGroup* group;
			/* setup vapidir and local packages */
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp15;
				GeeIterator* _tmp16;
				GeeIterator* _path_it;
				_tmp15 = NULL;
				_tmp16 = NULL;
				_path_it = (_tmp16 = gee_iterable_iterator ((GeeIterable*) (_tmp15 = vbf_group_get_include_dirs (group))), (_tmp15 == NULL) ? NULL : (_tmp15 = (gee_collection_object_unref (_tmp15), NULL)), _tmp16);
				while (gee_iterator_next (_path_it)) {
					char* path;
					path = (char*) gee_iterator_get (_path_it);
					vsc_parser_manager_add_path_to_vapi_search_dir (vsc_symbol_completion_get_parser (completion), path);
					path = (g_free (path), NULL);
				}
				(_path_it == NULL) ? NULL : (_path_it = (gee_collection_object_unref (_path_it), NULL));
			}
			{
				GeeList* _tmp17;
				GeeIterator* _tmp18;
				GeeIterator* _package_it;
				_tmp17 = NULL;
				_tmp18 = NULL;
				_package_it = (_tmp18 = gee_iterable_iterator ((GeeIterable*) (_tmp17 = vbf_group_get_packages (group))), (_tmp17 == NULL) ? NULL : (_tmp17 = (gee_collection_object_unref (_tmp17), NULL)), _tmp18);
				while (gee_iterator_next (_package_it)) {
					VbfPackage* package;
					package = (VbfPackage*) gee_iterator_get (_package_it);
					vsc_parser_manager_try_add_package (vsc_symbol_completion_get_parser (completion), package->id, TRUE);
					(package == NULL) ? NULL : (package = (g_object_unref (package), NULL));
				}
				(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
			}
			{
				GeeList* _tmp19;
				GeeIterator* _tmp20;
				GeeIterator* _target_it;
				_tmp19 = NULL;
				_tmp20 = NULL;
				_target_it = (_tmp20 = gee_iterable_iterator ((GeeIterable*) (_tmp19 = vbf_group_get_targets (group))), (_tmp19 == NULL) ? NULL : (_tmp19 = (gee_collection_object_unref (_tmp19), NULL)), _tmp20);
				while (gee_iterator_next (_target_it)) {
					VbfTarget* target;
					target = (VbfTarget*) gee_iterator_get (_target_it);
					{
						GeeList* _tmp21;
						GeeIterator* _tmp22;
						GeeIterator* _path_it;
						_tmp21 = NULL;
						_tmp22 = NULL;
						_path_it = (_tmp22 = gee_iterable_iterator ((GeeIterable*) (_tmp21 = vbf_target_get_include_dirs (target))), (_tmp21 == NULL) ? NULL : (_tmp21 = (gee_collection_object_unref (_tmp21), NULL)), _tmp22);
						while (gee_iterator_next (_path_it)) {
							char* path;
							path = (char*) gee_iterator_get (_path_it);
							vsc_parser_manager_add_path_to_vapi_search_dir (vsc_symbol_completion_get_parser (completion), path);
							path = (g_free (path), NULL);
						}
						(_path_it == NULL) ? NULL : (_path_it = (gee_collection_object_unref (_path_it), NULL));
					}
					{
						GeeList* _tmp23;
						GeeIterator* _tmp24;
						GeeIterator* _package_it;
						_tmp23 = NULL;
						_tmp24 = NULL;
						_package_it = (_tmp24 = gee_iterable_iterator ((GeeIterable*) (_tmp23 = vbf_target_get_packages (target))), (_tmp23 == NULL) ? NULL : (_tmp23 = (gee_collection_object_unref (_tmp23), NULL)), _tmp24);
						while (gee_iterator_next (_package_it)) {
							VbfPackage* package;
							package = (VbfPackage*) gee_iterator_get (_package_it);
							vsc_parser_manager_try_add_package (vsc_symbol_completion_get_parser (completion), package->id, TRUE);
							(package == NULL) ? NULL : (package = (g_object_unref (package), NULL));
						}
						(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	/* setup source files */
	{
		GeeList* _tmp25;
		GeeIterator* _tmp26;
		GeeIterator* _group_it;
		/* setup source files */
		_tmp25 = NULL;
		_tmp26 = NULL;
		_group_it = (_tmp26 = gee_iterable_iterator ((GeeIterable*) (_tmp25 = vbf_project_get_groups (project))), (_tmp25 == NULL) ? NULL : (_tmp25 = (gee_collection_object_unref (_tmp25), NULL)), _tmp26);
		/* setup source files */
		while (gee_iterator_next (_group_it)) {
			VbfGroup* group;
			/* setup source files */
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp27;
				GeeIterator* _tmp28;
				GeeIterator* _target_it;
				_tmp27 = NULL;
				_tmp28 = NULL;
				_target_it = (_tmp28 = gee_iterable_iterator ((GeeIterable*) (_tmp27 = vbf_group_get_targets (group))), (_tmp27 == NULL) ? NULL : (_tmp27 = (gee_collection_object_unref (_tmp27), NULL)), _tmp28);
				while (gee_iterator_next (_target_it)) {
					VbfTarget* target;
					target = (VbfTarget*) gee_iterator_get (_target_it);
					{
						GeeList* _tmp29;
						GeeIterator* _tmp30;
						GeeIterator* _source_it;
						_tmp29 = NULL;
						_tmp30 = NULL;
						_source_it = (_tmp30 = gee_iterable_iterator ((GeeIterable*) (_tmp29 = vbf_target_get_sources (target))), (_tmp29 == NULL) ? NULL : (_tmp29 = (gee_collection_object_unref (_tmp29), NULL)), _tmp30);
						while (gee_iterator_next (_source_it)) {
							VbfSource* source;
							source = (VbfSource*) gee_iterator_get (_source_it);
							if (((VbfFile*) source)->type == VBF_FILE_TYPES_VALA_SOURCE) {
								{
									vsc_parser_manager_add_source (vsc_symbol_completion_get_parser (completion), ((VbfFile*) source)->filename, &inner_error);
									if (inner_error != NULL) {
										goto __catch30_g_error;
										goto __finally30;
									}
								}
								goto __finally30;
								__catch30_g_error:
								{
									GError * err;
									err = inner_error;
									inner_error = NULL;
									{
										g_warning ("vtgplugin.vala:274: Error adding source %s: %s", ((VbfFile*) source)->filename, err->message);
										(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
									}
								}
								__finally30:
								if (inner_error != NULL) {
									(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
									(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
									(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
									(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
									(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
									(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
									(prj == NULL) ? NULL : (prj = (g_object_unref (prj), NULL));
									(completion == NULL) ? NULL : (completion = (g_object_unref (completion), NULL));
									(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
									g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
									g_clear_error (&inner_error);
									return;
								}
							}
							(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
						}
						(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	/* update the other project manager views */
	{
		GeeIterator* _instance_it;
		/* update the other project manager views */
		_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
		/* update the other project manager views */
		while (gee_iterator_next (_instance_it)) {
			VtgPluginInstance* instance;
			/* update the other project manager views */
			instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
			if (vtg_plugin_instance_get_project_manager_ui (instance) != sender) {
				vtg_project_view_add_project (vtg_project_manager_ui_get_project_view (vtg_plugin_instance_get_project_manager_ui (instance)), project);
			}
			(instance == NULL) ? NULL : (instance = (g_object_unref (instance), NULL));
		}
		(_instance_it == NULL) ? NULL : (_instance_it = (gee_collection_object_unref (_instance_it), NULL));
	}
	_tmp32 = NULL;
	_tmp31 = NULL;
	prj->completion = (_tmp32 = (_tmp31 = completion, (_tmp31 == NULL) ? NULL : g_object_ref (_tmp31)), (prj->completion == NULL) ? NULL : (prj->completion = (g_object_unref (prj->completion), NULL)), _tmp32);
	_tmp34 = NULL;
	_tmp33 = NULL;
	prj->project = (_tmp34 = (_tmp33 = project_manager, (_tmp33 == NULL) ? NULL : g_object_ref (_tmp33)), (prj->project == NULL) ? NULL : (prj->project = (g_object_unref (prj->project), NULL)), _tmp34);
	gee_collection_add ((GeeCollection*) self->priv->_projects, prj);
	vsc_parser_manager_resume_parsing (vsc_symbol_completion_get_parser (completion));
	(prj == NULL) ? NULL : (prj = (g_object_unref (prj), NULL));
	(completion == NULL) ? NULL : (completion = (g_object_unref (completion), NULL));
	(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
}


VtgPlugin* vtg_plugin_construct (GType object_type) {
	VtgPlugin * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VtgPlugin* vtg_plugin_new (void) {
	return vtg_plugin_construct (VTG_TYPE_PLUGIN);
}


GeeList* vtg_plugin_get_projects (VtgPlugin* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_projects;
}


VtgConfiguration* vtg_plugin_get_config (VtgPlugin* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_config;
}


static void _vtg_plugin_on_configuration_property_changed_g_object_notify (VtgConfiguration* _sender, GParamSpec* pspec, gpointer self) {
	vtg_plugin_on_configuration_property_changed (self, _sender, pspec);
}


static GObject * vtg_plugin_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgPluginClass * klass;
	GObjectClass * parent_class;
	VtgPlugin * self;
	klass = VTG_PLUGIN_CLASS (g_type_class_peek (VTG_TYPE_PLUGIN));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_PLUGIN (obj);
	{
		VtgConfiguration* _tmp0;
		_tmp0 = NULL;
		self->priv->_config = (_tmp0 = vtg_configuration_new (), (self->priv->_config == NULL) ? NULL : (self->priv->_config = (g_object_unref (self->priv->_config), NULL)), _tmp0);
		g_signal_connect_object ((GObject*) self->priv->_config, "notify", (GCallback) _vtg_plugin_on_configuration_property_changed_g_object_notify, self, 0);
		bindtextdomain (GETTEXT_PACKAGE, NULL);
	}
	return obj;
}


static void vtg_plugin_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgPlugin * self;
	gpointer boxed;
	self = VTG_PLUGIN (object);
	switch (property_id) {
		case VTG_PLUGIN_PROJECTS:
		g_value_set_object (value, vtg_plugin_get_projects (self));
		break;
		case VTG_PLUGIN_CONFIG:
		g_value_set_object (value, vtg_plugin_get_config (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_plugin_class_init (VtgPluginClass * klass) {
	vtg_plugin_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgPluginPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_plugin_get_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_plugin_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_plugin_finalize;
	GEDIT_PLUGIN_CLASS (klass)->activate = vtg_plugin_real_activate;
	GEDIT_PLUGIN_CLASS (klass)->deactivate = vtg_plugin_real_deactivate;
	GEDIT_PLUGIN_CLASS (klass)->is_configurable = vtg_plugin_real_is_configurable;
	GEDIT_PLUGIN_CLASS (klass)->create_configure_dialog = vtg_plugin_real_create_configure_dialog;
	GEDIT_PLUGIN_CLASS (klass)->update_ui = vtg_plugin_real_update_ui;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PLUGIN_PROJECTS, gee_param_spec_collection_object ("projects", "projects", "projects", GEE_TYPE_LIST, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PLUGIN_CONFIG, g_param_spec_object ("config", "config", "config", VTG_TYPE_CONFIGURATION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vtg_plugin_instance_init (VtgPlugin * self) {
	self->priv = VTG_PLUGIN_GET_PRIVATE (self);
	self->priv->_instances = (GeeList*) gee_array_list_new (VTG_TYPE_PLUGIN_INSTANCE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->_projects = (GeeList*) gee_array_list_new (VTG_TYPE_PROJECT_DESCRIPTOR, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->_config = NULL;
	self->priv->_default_project = NULL;
}


static void vtg_plugin_finalize (GObject* obj) {
	VtgPlugin * self;
	self = VTG_PLUGIN (obj);
	{
		vtg_plugin_deactivate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL);
	}
	(self->priv->_instances == NULL) ? NULL : (self->priv->_instances = (gee_collection_object_unref (self->priv->_instances), NULL));
	(self->priv->_projects == NULL) ? NULL : (self->priv->_projects = (gee_collection_object_unref (self->priv->_projects), NULL));
	(self->priv->_config == NULL) ? NULL : (self->priv->_config = (g_object_unref (self->priv->_config), NULL));
	(self->priv->_default_project == NULL) ? NULL : (self->priv->_default_project = (g_object_unref (self->priv->_default_project), NULL));
	G_OBJECT_CLASS (vtg_plugin_parent_class)->finalize (obj);
}


static GType vtg_plugin_type_id = 0;
GType vtg_plugin_get_type (void) {
	return vtg_plugin_type_id;
}


GType vtg_plugin_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgPluginClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_plugin_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgPlugin), 0, (GInstanceInitFunc) vtg_plugin_instance_init, NULL };
	vtg_plugin_type_id = g_type_module_register_type (module, GEDIT_TYPE_PLUGIN, "VtgPlugin", &g_define_type_info, 0);
	return vtg_plugin_type_id;
}


GType register_gedit_plugin (GTypeModule* module) {
	g_return_val_if_fail (module != NULL, 0UL);
	vtg_bracket_completion_register_type (module);
	vtg_symbol_completion_provider_register_type (module);
	vtg_symbol_completion_trigger_register_type (module);
	vtg_symbol_completion_helper_register_type (module);
	vtg_project_manager_ui_register_type (module);
	vtg_project_descriptor_register_type (module);
	vtg_project_view_register_type (module);
	vtg_project_builder_register_type (module);
	vtg_project_executer_register_type (module);
	vtg_project_executer_dialog_register_type (module);
	vtg_output_view_register_type (module);
	vtg_process_watch_info_register_type (module);
	vtg_build_log_view_register_type (module);
	vtg_project_manager_register_type (module);
	vtg_filtered_list_dialog_register_type (module);
	vtg_vcs_backends_item_register_type (module);
	vtg_vcs_backends_git_register_type (module);
	vtg_vcs_backends_bzr_register_type (module);
	vtg_vcs_backends_svn_register_type (module);
	vtg_change_log_register_type (module);
	vtg_interaction_parameters_dialog_register_type (module);
	vtg_configuration_register_type (module);
	vtg_source_bookmark_register_type (module);
	vtg_source_bookmarks_register_type (module);
	vtg_plugin_instance_register_type (module);
	vtg_plugin_register_type (module);
	return VTG_TYPE_PLUGIN;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




