/*
 *  vtgplugininstance.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#ifndef __VTGPLUGININSTANCE_H__
#define __VTGPLUGININSTANCE_H__

#include <glib.h>
#include <glib-object.h>
#include <gedit/gedit-window.h>
#include <gedit/gedit-tab.h>
#include <stdlib.h>
#include <string.h>
#include <gedit/gedit-view.h>
#include <gedit/gedit-document.h>
#include "vtgbracketcompletion.h"
#include "vtgprojectdescriptor.h"

G_BEGIN_DECLS


#define VTG_TYPE_PLUGIN_INSTANCE (vtg_plugin_instance_get_type ())
#define VTG_PLUGIN_INSTANCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstance))
#define VTG_PLUGIN_INSTANCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstanceClass))
#define VTG_IS_PLUGIN_INSTANCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PLUGIN_INSTANCE))
#define VTG_IS_PLUGIN_INSTANCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PLUGIN_INSTANCE))
#define VTG_PLUGIN_INSTANCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstanceClass))

typedef struct _VtgPluginInstancePrivate VtgPluginInstancePrivate;

struct _VtgPluginInstance {
	GObject parent_instance;
	VtgPluginInstancePrivate * priv;
	VtgPlugin* plugin;
};

struct _VtgPluginInstanceClass {
	GObjectClass parent_class;
};


VtgPluginInstance* vtg_plugin_instance_construct (GType object_type, VtgPlugin* plugin, GeditWindow* window);
VtgPluginInstance* vtg_plugin_instance_new (VtgPlugin* plugin, GeditWindow* window);
void vtg_plugin_instance_initialize_views (VtgPluginInstance* self);
void vtg_plugin_instance_initialize_view (VtgPluginInstance* self, VtgProjectDescriptor* project, GeditView* view);
void vtg_plugin_instance_initialize_document (VtgPluginInstance* self, GeditDocument* doc);
void vtg_plugin_instance_uninitialize_view (VtgPluginInstance* self, GeditView* view);
void vtg_plugin_instance_activate_bracket (VtgPluginInstance* self, GeditView* view);
void vtg_plugin_instance_deactivate_bracket (VtgPluginInstance* self, VtgBracketCompletion* bc);
void vtg_plugin_instance_activate_symbol (VtgPluginInstance* self, VtgProjectDescriptor* project, GeditView* view);
void vtg_plugin_instance_deactivate_symbol (VtgPluginInstance* self, VtgSymbolCompletionHelper* sc);
void vtg_plugin_instance_deactivate_symbols (VtgPluginInstance* self);
void vtg_plugin_instance_deactivate_brackets (VtgPluginInstance* self);
gboolean vtg_plugin_instance_bcs_contains (VtgPluginInstance* self, GeditView* view);
VtgBracketCompletion* vtg_plugin_instance_bcs_find_from_view (VtgPluginInstance* self, GeditView* view);
gboolean vtg_plugin_instance_scs_contains (VtgPluginInstance* self, GeditView* view);
VtgSymbolCompletionHelper* vtg_plugin_instance_scs_find_from_view (VtgPluginInstance* self, GeditView* view);
void vtg_plugin_instance_uninitialize_document (VtgPluginInstance* self, GeditDocument* doc);
GeditTab* vtg_plugin_instance_activate_uri (VtgPluginInstance* self, const char* uri, gint line, gint col);
void vtg_plugin_instance_activate_display_name (VtgPluginInstance* self, const char* display_name, gint line, gint col);
VtgOutputView* vtg_plugin_instance_get_output_view (VtgPluginInstance* self);
VtgProjectManagerUi* vtg_plugin_instance_get_project_manager_ui (VtgPluginInstance* self);
GeditWindow* vtg_plugin_instance_get_window (VtgPluginInstance* self);
GType vtg_plugin_instance_get_type (void);
GType vtg_plugin_instance_register_type (GTypeModule * module);


G_END_DECLS

#endif
