/*
 *  vtgprojectbuilder.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#ifndef __VTGPROJECTBUILDER_H__
#define __VTGPROJECTBUILDER_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include "vtgplugininstance.h"
#include "vtgprojectmanager.h"
#include "vtgbuildlogview.h"

G_BEGIN_DECLS


#define VTG_TYPE_PROJECT_BUILDER (vtg_project_builder_get_type ())
#define VTG_PROJECT_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_BUILDER, VtgProjectBuilder))
#define VTG_PROJECT_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_BUILDER, VtgProjectBuilderClass))
#define VTG_IS_PROJECT_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_BUILDER))
#define VTG_IS_PROJECT_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_BUILDER))
#define VTG_PROJECT_BUILDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_BUILDER, VtgProjectBuilderClass))

typedef struct _VtgProjectBuilder VtgProjectBuilder;
typedef struct _VtgProjectBuilderClass VtgProjectBuilderClass;
typedef struct _VtgProjectBuilderPrivate VtgProjectBuilderPrivate;

struct _VtgProjectBuilder {
	GObject parent_instance;
	VtgProjectBuilderPrivate * priv;
};

struct _VtgProjectBuilderClass {
	GObjectClass parent_class;
};


VtgProjectBuilder* vtg_project_builder_construct (GType object_type, VtgPluginInstance* plugin_instance);
VtgProjectBuilder* vtg_project_builder_new (VtgPluginInstance* plugin_instance);
gboolean vtg_project_builder_compile_file (VtgProjectBuilder* self, const char* filename, const char* params);
gboolean vtg_project_builder_build (VtgProjectBuilder* self, VtgProjectManager* project_manager, const char* params);
gboolean vtg_project_builder_configure (VtgProjectBuilder* self, VtgProjectManager* project_manager, const char* params);
gboolean vtg_project_builder_clean (VtgProjectBuilder* self, VtgProjectManager* project_manager, gboolean vala_stamp);
void vtg_project_builder_next_error (VtgProjectBuilder* self);
void vtg_project_builder_previous_error (VtgProjectBuilder* self);
void vtg_project_builder_stop_build (VtgProjectBuilder* self);
VtgPluginInstance* vtg_project_builder_get_plugin_instance (VtgProjectBuilder* self);
VtgBuildLogView* vtg_project_builder_get_error_pane (VtgProjectBuilder* self);
gboolean vtg_project_builder_get_is_building (VtgProjectBuilder* self);
GType vtg_project_builder_get_type (void);
GType vtg_project_builder_register_type (GTypeModule * module);


G_END_DECLS

#endif
