/*
 *  vtgprojectmanager.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanager.h"
#include <gee/arraylist.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <vbfgroup.h>
#include <vbftarget.h>
#include <vbffile.h>
#include <am/vbfamprojectmanager.h>
#include <vbfiprojectmanager.h>
#include <gee/collection.h>
#include <glib/gstdio.h>
#include <glib/gi18n-lib.h>
#include <vbfmodule.h>
#include <vbfpackage.h>
#include "vtgvcsbackendsgit.h"
#include "vtgvcsbackendsivcs.h"
#include "vtgvcsbackendsbzr.h"
#include "vtgvcsbackendssvn.h"




struct _VtgProjectManagerPrivate {
	VbfProject* _project;
	GtkTreeStore* _model;
	gboolean in_update;
};

#define VTG_PROJECT_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerPrivate))
enum  {
	VTG_PROJECT_MANAGER_DUMMY_PROPERTY,
	VTG_PROJECT_MANAGER_MODEL,
	VTG_PROJECT_MANAGER_PROJECT
};
static void _vtg_project_manager_on_project_updated_vbf_project_updated (VbfProject* _sender, gpointer self);
static void vtg_project_manager_vcs_test (VtgProjectManager* self, const char* filename);
static void vtg_project_manager_on_project_updated (VtgProjectManager* self, VbfProject* sender);
static gint _vtg_project_manager_sort_model_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self);
static void vtg_project_manager_build_tree_model (VtgProjectManager* self);
static void vtg_project_manager_parse_project (VtgProjectManager* self);
static gint vtg_project_manager_sort_model (VtgProjectManager* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b);
static gpointer vtg_project_manager_parent_class = NULL;
static void vtg_project_manager_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



gboolean vtg_project_manager_contains_file (VtgProjectManager* self, const char* uri) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (uri != NULL, FALSE);
	{
		GeeList* _tmp0;
		GeeIterator* _tmp1;
		GeeIterator* _group_it;
		_tmp0 = NULL;
		_tmp1 = NULL;
		_group_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vbf_project_get_groups (self->priv->_project))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
		while (gee_iterator_next (_group_it)) {
			VbfGroup* group;
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp2;
				GeeIterator* _tmp3;
				GeeIterator* _target_it;
				_tmp2 = NULL;
				_tmp3 = NULL;
				_target_it = (_tmp3 = gee_iterable_iterator ((GeeIterable*) (_tmp2 = vbf_group_get_targets (group))), (_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)), _tmp3);
				while (gee_iterator_next (_target_it)) {
					VbfTarget* target;
					target = (VbfTarget*) gee_iterator_get (_target_it);
					{
						GeeList* _tmp4;
						GeeIterator* _tmp5;
						GeeIterator* _source_it;
						_tmp4 = NULL;
						_tmp5 = NULL;
						_source_it = (_tmp5 = gee_iterable_iterator ((GeeIterable*) (_tmp4 = vbf_target_get_sources (target))), (_tmp4 == NULL) ? NULL : (_tmp4 = (gee_collection_object_unref (_tmp4), NULL)), _tmp5);
						while (gee_iterator_next (_source_it)) {
							VbfSource* source;
							source = (VbfSource*) gee_iterator_get (_source_it);
							if (_vala_strcmp0 (((VbfFile*) source)->uri, uri) == 0) {
								gboolean _tmp6;
								return (_tmp6 = TRUE, (source == NULL) ? NULL : (source = (g_object_unref (source), NULL)), (_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL)), (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), (_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL)), (group == NULL) ? NULL : (group = (g_object_unref (group), NULL)), (_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL)), _tmp6);
							}
							(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
						}
						(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
					}
					{
						GeeList* _tmp7;
						GeeIterator* _tmp8;
						GeeIterator* _file_it;
						_tmp7 = NULL;
						_tmp8 = NULL;
						_file_it = (_tmp8 = gee_iterable_iterator ((GeeIterable*) (_tmp7 = vbf_target_get_files (target))), (_tmp7 == NULL) ? NULL : (_tmp7 = (gee_collection_object_unref (_tmp7), NULL)), _tmp8);
						while (gee_iterator_next (_file_it)) {
							VbfFile* file;
							file = (VbfFile*) gee_iterator_get (_file_it);
							if (_vala_strcmp0 (file->uri, uri) == 0) {
								gboolean _tmp9;
								return (_tmp9 = TRUE, (file == NULL) ? NULL : (file = (g_object_unref (file), NULL)), (_file_it == NULL) ? NULL : (_file_it = (gee_collection_object_unref (_file_it), NULL)), (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), (_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL)), (group == NULL) ? NULL : (group = (g_object_unref (group), NULL)), (_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL)), _tmp9);
							}
							(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
						}
						(_file_it == NULL) ? NULL : (_file_it = (gee_collection_object_unref (_file_it), NULL));
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	return FALSE;
}


VbfSource* vtg_project_manager_get_source_file_from_uri (VtgProjectManager* self, const char* uri) {
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (uri != NULL, NULL);
	{
		GeeList* _tmp0;
		GeeIterator* _tmp1;
		GeeIterator* _group_it;
		_tmp0 = NULL;
		_tmp1 = NULL;
		_group_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vbf_project_get_groups (self->priv->_project))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
		while (gee_iterator_next (_group_it)) {
			VbfGroup* group;
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp2;
				GeeIterator* _tmp3;
				GeeIterator* _target_it;
				_tmp2 = NULL;
				_tmp3 = NULL;
				_target_it = (_tmp3 = gee_iterable_iterator ((GeeIterable*) (_tmp2 = vbf_group_get_targets (group))), (_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)), _tmp3);
				while (gee_iterator_next (_target_it)) {
					VbfTarget* target;
					target = (VbfTarget*) gee_iterator_get (_target_it);
					{
						GeeList* _tmp4;
						GeeIterator* _tmp5;
						GeeIterator* _source_it;
						_tmp4 = NULL;
						_tmp5 = NULL;
						_source_it = (_tmp5 = gee_iterable_iterator ((GeeIterable*) (_tmp4 = vbf_target_get_sources (target))), (_tmp4 == NULL) ? NULL : (_tmp4 = (gee_collection_object_unref (_tmp4), NULL)), _tmp5);
						while (gee_iterator_next (_source_it)) {
							VbfSource* source;
							source = (VbfSource*) gee_iterator_get (_source_it);
							if (_vala_strcmp0 (((VbfFile*) source)->uri, uri) == 0) {
								VbfSource* _tmp6;
								_tmp6 = NULL;
								return (_tmp6 = source, (_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL)), (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), (_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL)), (group == NULL) ? NULL : (group = (g_object_unref (group), NULL)), (_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL)), _tmp6);
							}
							(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
						}
						(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	return NULL;
}


gboolean vtg_project_manager_contains_vala_source_file (VtgProjectManager* self, const char* uri) {
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (uri != NULL, FALSE);
	{
		GeeIterator* _source_it;
		_source_it = gee_iterable_iterator ((GeeIterable*) self->all_vala_sources);
		while (gee_iterator_next (_source_it)) {
			VbfSource* source;
			source = (VbfSource*) gee_iterator_get (_source_it);
			if (_vala_strcmp0 (((VbfFile*) source)->uri, uri) == 0) {
				gboolean _tmp0;
				return (_tmp0 = TRUE, (source == NULL) ? NULL : (source = (g_object_unref (source), NULL)), (_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL)), _tmp0);
			}
			(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
		}
		(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
	}
	return FALSE;
}


char* vtg_project_manager_source_uri_for_name (VtgProjectManager* self, const char* name) {
	char** _tmp1;
	gint name_parts_size;
	gint name_parts_length1;
	char** _tmp0;
	char** name_parts;
	char* _tmp14;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	name_parts = (_tmp1 = _tmp0 = g_strsplit (name, "/", 0), name_parts_length1 = _vala_array_length (_tmp0), name_parts_size = name_parts_length1, _tmp1);
	{
		GeeList* _tmp2;
		GeeIterator* _tmp3;
		GeeIterator* _group_it;
		_tmp2 = NULL;
		_tmp3 = NULL;
		_group_it = (_tmp3 = gee_iterable_iterator ((GeeIterable*) (_tmp2 = vbf_project_get_groups (self->priv->_project))), (_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)), _tmp3);
		while (gee_iterator_next (_group_it)) {
			VbfGroup* group;
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp4;
				GeeIterator* _tmp5;
				GeeIterator* _target_it;
				_tmp4 = NULL;
				_tmp5 = NULL;
				_target_it = (_tmp5 = gee_iterable_iterator ((GeeIterable*) (_tmp4 = vbf_group_get_targets (group))), (_tmp4 == NULL) ? NULL : (_tmp4 = (gee_collection_object_unref (_tmp4), NULL)), _tmp5);
				while (gee_iterator_next (_target_it)) {
					VbfTarget* target;
					target = (VbfTarget*) gee_iterator_get (_target_it);
					{
						GeeList* _tmp6;
						GeeIterator* _tmp7;
						GeeIterator* _source_it;
						_tmp6 = NULL;
						_tmp7 = NULL;
						_source_it = (_tmp7 = gee_iterable_iterator ((GeeIterable*) (_tmp6 = vbf_target_get_sources (target))), (_tmp6 == NULL) ? NULL : (_tmp6 = (gee_collection_object_unref (_tmp6), NULL)), _tmp7);
						while (gee_iterator_next (_source_it)) {
							VbfSource* source;
							source = (VbfSource*) gee_iterator_get (_source_it);
							if (name_parts_length1 == 1) {
								if (_vala_strcmp0 (((VbfFile*) source)->name, name) == 0) {
									const char* _tmp8;
									char* _tmp9;
									_tmp8 = NULL;
									_tmp9 = NULL;
									return (_tmp9 = (_tmp8 = ((VbfFile*) source)->uri, (_tmp8 == NULL) ? NULL : g_strdup (_tmp8)), (source == NULL) ? NULL : (source = (g_object_unref (source), NULL)), (_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL)), (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), (_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL)), (group == NULL) ? NULL : (group = (g_object_unref (group), NULL)), (_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL)), name_parts = (_vala_array_free (name_parts, name_parts_length1, (GDestroyNotify) g_free), NULL), _tmp9);
								}
							} else {
								if (((VbfFile*) source)->uri != NULL) {
									char** _tmp11;
									gint src_parts_size;
									gint src_parts_length1;
									char** _tmp10;
									char** src_parts;
									_tmp11 = NULL;
									_tmp10 = NULL;
									src_parts = (_tmp11 = _tmp10 = g_strsplit (((VbfFile*) source)->uri, "/", 0), src_parts_length1 = _vala_array_length (_tmp10), src_parts_size = src_parts_length1, _tmp11);
									if (name_parts_length1 <= src_parts_length1) {
										gboolean equals;
										equals = TRUE;
										{
											gint idx;
											idx = 0;
											for (; idx < name_parts_length1; idx++) {
												if (_vala_strcmp0 (src_parts[src_parts_length1 - idx], name_parts[name_parts_length1 - idx]) != 0) {
													equals = FALSE;
													break;
												}
											}
										}
										if (equals) {
											const char* _tmp12;
											char* _tmp13;
											_tmp12 = NULL;
											_tmp13 = NULL;
											return (_tmp13 = (_tmp12 = ((VbfFile*) source)->uri, (_tmp12 == NULL) ? NULL : g_strdup (_tmp12)), src_parts = (_vala_array_free (src_parts, src_parts_length1, (GDestroyNotify) g_free), NULL), (source == NULL) ? NULL : (source = (g_object_unref (source), NULL)), (_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL)), (target == NULL) ? NULL : (target = (g_object_unref (target), NULL)), (_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL)), (group == NULL) ? NULL : (group = (g_object_unref (group), NULL)), (_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL)), name_parts = (_vala_array_free (name_parts, name_parts_length1, (GDestroyNotify) g_free), NULL), _tmp13);
										}
									}
									src_parts = (_vala_array_free (src_parts, src_parts_length1, (GDestroyNotify) g_free), NULL);
								}
							}
							(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
						}
						(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	_tmp14 = NULL;
	return (_tmp14 = NULL, name_parts = (_vala_array_free (name_parts, name_parts_length1, (GDestroyNotify) g_free), NULL), _tmp14);
}


static void _vtg_project_manager_on_project_updated_vbf_project_updated (VbfProject* _sender, gpointer self) {
	vtg_project_manager_on_project_updated (self, _sender);
}


gboolean vtg_project_manager_open (VtgProjectManager* self, const char* project_filename, GError** error) {
	GError * inner_error;
	VbfIProjectManager* pm;
	gboolean res;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (project_filename != NULL, FALSE);
	inner_error = NULL;
	pm = (VbfIProjectManager*) vbf_am_project_manager_new ();
	res = vbf_iproject_manager_probe (pm, project_filename);
	if (res) {
		VbfProject* _tmp0;
		gboolean _tmp2;
		_tmp0 = NULL;
		self->priv->_project = (_tmp0 = vbf_iproject_manager_open (pm, project_filename), (self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL)), _tmp0);
		if (self->priv->_project == NULL) {
			gboolean _tmp1;
			return (_tmp1 = FALSE, (pm == NULL) ? NULL : (pm = (g_object_unref (pm), NULL)), _tmp1);
		}
		vtg_project_manager_parse_project (self);
		vtg_project_manager_build_tree_model (self);
		vtg_project_manager_vcs_test (self, project_filename);
		g_signal_connect_object (self->priv->_project, "updated", (GCallback) _vtg_project_manager_on_project_updated_vbf_project_updated, self, 0);
		return (_tmp2 = TRUE, (pm == NULL) ? NULL : (pm = (g_object_unref (pm), NULL)), _tmp2);
	} else {
		inner_error = g_error_new_literal (VTG_PROJECT_MANAGER_ERROR, VTG_PROJECT_MANAGER_ERROR_NO_BACKEND, _ ("Can't load project, no suitable backend found"));
		if (inner_error != NULL) {
			g_propagate_error (error, inner_error);
			(pm == NULL) ? NULL : (pm = (g_object_unref (pm), NULL));
			return FALSE;
		}
	}
	(pm == NULL) ? NULL : (pm = (g_object_unref (pm), NULL));
}


static void vtg_project_manager_vcs_test (VtgProjectManager* self, const char* filename) {
	VtgVcsBackendsIVcs* backend;
	g_return_if_fail (self != NULL);
	g_return_if_fail (filename != NULL);
	/*test if the project is under some known revision control system*/
	backend = (VtgVcsBackendsIVcs*) vtg_vcs_backends_git_new ();
	self->vcs_type = VTG_VCS_TYPES_NONE;
	if (vtg_vcs_backends_ivcs_test (backend, filename)) {
		self->vcs_type = VTG_VCS_TYPES_GIT;
	} else {
		VtgVcsBackendsIVcs* _tmp0;
		_tmp0 = NULL;
		backend = (_tmp0 = (VtgVcsBackendsIVcs*) vtg_vcs_backends_bzr_new (), (backend == NULL) ? NULL : (backend = (g_object_unref (backend), NULL)), _tmp0);
		if (vtg_vcs_backends_ivcs_test (backend, filename)) {
			self->vcs_type = VTG_VCS_TYPES_BZR;
		} else {
			VtgVcsBackendsIVcs* _tmp1;
			_tmp1 = NULL;
			backend = (_tmp1 = (VtgVcsBackendsIVcs*) vtg_vcs_backends_svn_new (), (backend == NULL) ? NULL : (backend = (g_object_unref (backend), NULL)), _tmp1);
			if (vtg_vcs_backends_ivcs_test (backend, filename)) {
				self->vcs_type = VTG_VCS_TYPES_SVN;
			}
		}
	}
	(backend == NULL) ? NULL : (backend = (g_object_unref (backend), NULL));
}


void vtg_project_manager_close (VtgProjectManager* self) {
	GtkTreeStore* _tmp0;
	VbfProject* _tmp1;
	g_return_if_fail (self != NULL);
	gee_collection_clear ((GeeCollection*) self->exec_targets);
	gee_collection_clear ((GeeCollection*) self->all_vala_sources);
	_tmp0 = NULL;
	self->priv->_model = (_tmp0 = NULL, (self->priv->_model == NULL) ? NULL : (self->priv->_model = (g_object_unref (self->priv->_model), NULL)), _tmp0);
	_tmp1 = NULL;
	self->priv->_project = (_tmp1 = NULL, (self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL)), _tmp1);
}


static void vtg_project_manager_on_project_updated (VtgProjectManager* self, VbfProject* sender) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	if (self->priv->in_update) {
		return;
	}
	self->priv->in_update = TRUE;
	vtg_project_manager_parse_project (self);
	vtg_project_manager_build_tree_model (self);
	g_signal_emit_by_name (self, "updated");
	self->priv->in_update = FALSE;
}


static gint _vtg_project_manager_sort_model_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self) {
	return vtg_project_manager_sort_model (self, model, a, b);
}


static void vtg_project_manager_build_tree_model (VtgProjectManager* self) {
	GtkTreeIter project_iter = {0};
	GtkTreeIter modules_iter = {0};
	GtkTreeIter groups_iter = {0};
	GtkTreeStore* _tmp0;
	char* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	self->priv->_model = (_tmp0 = gtk_tree_store_new (5, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_OBJECT, G_TYPE_STRING, NULL), (self->priv->_model == NULL) ? NULL : (self->priv->_model = (g_object_unref (self->priv->_model), NULL)), _tmp0);
	gtk_tree_store_append (self->priv->_model, &project_iter, NULL);
	_tmp1 = NULL;
	gtk_tree_store_set (self->priv->_model, &project_iter, 0, GTK_STOCK_DIRECTORY, 1, _tmp1 = g_strdup_printf ("%s - %s", self->priv->_project->name, self->priv->_project->version), 2, "project-root", 4, "", -1);
	_tmp1 = (g_free (_tmp1), NULL);
	gtk_tree_store_append (self->priv->_model, &modules_iter, &project_iter);
	gtk_tree_store_set (self->priv->_model, &modules_iter, 0, GTK_STOCK_DIRECTORY, 1, _ ("References"), 2, "project-reference", 4, "1", -1);
	{
		GeeList* _tmp2;
		GeeIterator* _tmp3;
		GeeIterator* _module_it;
		_tmp2 = NULL;
		_tmp3 = NULL;
		_module_it = (_tmp3 = gee_iterable_iterator ((GeeIterable*) (_tmp2 = vbf_project_get_modules (self->priv->_project))), (_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)), _tmp3);
		while (gee_iterator_next (_module_it)) {
			VbfModule* module;
			GtkTreeIter module_iter = {0};
			module = (VbfModule*) gee_iterator_get (_module_it);
			gtk_tree_store_append (self->priv->_model, &module_iter, &modules_iter);
			gtk_tree_store_set (self->priv->_model, &module_iter, 0, GTK_STOCK_DIRECTORY, 1, module->name, 2, module->id, 3, module, 4, module->name, -1);
			{
				GeeList* _tmp4;
				GeeIterator* _tmp5;
				GeeIterator* _package_it;
				_tmp4 = NULL;
				_tmp5 = NULL;
				_package_it = (_tmp5 = gee_iterable_iterator ((GeeIterable*) (_tmp4 = vbf_module_get_packages (module))), (_tmp4 == NULL) ? NULL : (_tmp4 = (gee_collection_object_unref (_tmp4), NULL)), _tmp5);
				while (gee_iterator_next (_package_it)) {
					VbfPackage* package;
					GtkTreeIter package_iter = {0};
					package = (VbfPackage*) gee_iterator_get (_package_it);
					gtk_tree_store_append (self->priv->_model, &package_iter, &module_iter);
					gtk_tree_store_set (self->priv->_model, &package_iter, 0, GTK_STOCK_FILE, 1, package->name, 2, package->id, 3, package, 4, package->name, -1);
					(package == NULL) ? NULL : (package = (g_object_unref (package), NULL));
				}
				(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
			}
			(module == NULL) ? NULL : (module = (g_object_unref (module), NULL));
		}
		(_module_it == NULL) ? NULL : (_module_it = (gee_collection_object_unref (_module_it), NULL));
	}
	gtk_tree_store_append (self->priv->_model, &groups_iter, &project_iter);
	gtk_tree_store_set (self->priv->_model, &groups_iter, 0, GTK_STOCK_DIRECTORY, 1, _ ("Files"), 2, "project-files", 4, "2", -1);
	{
		GeeList* _tmp6;
		GeeIterator* _tmp7;
		GeeIterator* _group_it;
		_tmp6 = NULL;
		_tmp7 = NULL;
		_group_it = (_tmp7 = gee_iterable_iterator ((GeeIterable*) (_tmp6 = vbf_project_get_groups (self->priv->_project))), (_tmp6 == NULL) ? NULL : (_tmp6 = (gee_collection_object_unref (_tmp6), NULL)), _tmp7);
		while (gee_iterator_next (_group_it)) {
			VbfGroup* group;
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp8;
				GeeIterator* _tmp9;
				GeeIterator* _target_it;
				_tmp8 = NULL;
				_tmp9 = NULL;
				_target_it = (_tmp9 = gee_iterable_iterator ((GeeIterable*) (_tmp8 = vbf_group_get_targets (group))), (_tmp8 == NULL) ? NULL : (_tmp8 = (gee_collection_object_unref (_tmp8), NULL)), _tmp9);
				while (gee_iterator_next (_target_it)) {
					VbfTarget* target;
					gboolean _tmp10;
					target = (VbfTarget*) gee_iterator_get (_target_it);
					_tmp10 = FALSE;
					if (vbf_target_has_sources_of_type (target, VBF_FILE_TYPES_VALA_SOURCE)) {
						_tmp10 = TRUE;
					} else {
						GeeList* _tmp11;
						_tmp11 = NULL;
						_tmp10 = gee_collection_get_size ((GeeCollection*) (_tmp11 = vbf_target_get_files (target))) > 0;
						(_tmp11 == NULL) ? NULL : (_tmp11 = (gee_collection_object_unref (_tmp11), NULL));
					}
					if (_tmp10) {
						GtkTreeIter target_iter;
						gboolean target_added;
						target_iter = groups_iter;
						target_added = FALSE;
						{
							GeeList* _tmp12;
							GeeIterator* _tmp13;
							GeeIterator* _source_it;
							_tmp12 = NULL;
							_tmp13 = NULL;
							_source_it = (_tmp13 = gee_iterable_iterator ((GeeIterable*) (_tmp12 = vbf_target_get_sources (target))), (_tmp12 == NULL) ? NULL : (_tmp12 = (gee_collection_object_unref (_tmp12), NULL)), _tmp13);
							while (gee_iterator_next (_source_it)) {
								VbfSource* source;
								gboolean _tmp14;
								gboolean _tmp15;
								gboolean _tmp16;
								GtkTreeIter source_iter = {0};
								source = (VbfSource*) gee_iterator_get (_source_it);
								_tmp14 = FALSE;
								_tmp15 = FALSE;
								_tmp16 = FALSE;
								if (g_str_has_prefix (((VbfFile*) source)->name, ".")) {
									_tmp16 = TRUE;
								} else {
									_tmp16 = g_str_has_suffix (((VbfFile*) source)->name, ".c");
								}
								if (_tmp16) {
									_tmp15 = TRUE;
								} else {
									_tmp15 = g_str_has_suffix (((VbfFile*) source)->name, ".h");
								}
								if (_tmp15) {
									_tmp14 = TRUE;
								} else {
									_tmp14 = g_str_has_suffix (((VbfFile*) source)->name, ".stamp");
								}
								if (_tmp14) {
									(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
									continue;
								}
								if (!target_added) {
									gtk_tree_store_append (self->priv->_model, &target_iter, &groups_iter);
									gtk_tree_store_set (self->priv->_model, &target_iter, 0, GTK_STOCK_DIRECTORY, 1, group->name, 2, target->id, 3, target, 4, group->name, -1);
									target_added = TRUE;
								}
								gtk_tree_store_append (self->priv->_model, &source_iter, &target_iter);
								gtk_tree_store_set (self->priv->_model, &source_iter, 0, GTK_STOCK_FILE, 1, ((VbfFile*) source)->name, 2, ((VbfFile*) source)->uri, 3, source, 4, ((VbfFile*) source)->name, -1);
								(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
							}
							(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
						}
						{
							GeeList* _tmp17;
							GeeIterator* _tmp18;
							GeeIterator* _file_it;
							_tmp17 = NULL;
							_tmp18 = NULL;
							_file_it = (_tmp18 = gee_iterable_iterator ((GeeIterable*) (_tmp17 = vbf_target_get_files (target))), (_tmp17 == NULL) ? NULL : (_tmp17 = (gee_collection_object_unref (_tmp17), NULL)), _tmp18);
							while (gee_iterator_next (_file_it)) {
								VbfFile* file;
								GtkTreeIter file_iter = {0};
								file = (VbfFile*) gee_iterator_get (_file_it);
								if (!target_added) {
									gtk_tree_store_append (self->priv->_model, &target_iter, &groups_iter);
									gtk_tree_store_set (self->priv->_model, &target_iter, 0, GTK_STOCK_DIRECTORY, 1, group->name, 2, target->id, 3, target, 4, group->name, -1);
									target_added = TRUE;
								}
								gtk_tree_store_append (self->priv->_model, &file_iter, &target_iter);
								gtk_tree_store_set (self->priv->_model, &file_iter, 0, GTK_STOCK_FILE, 1, file->name, 2, file->uri, 3, file, 4, file->name, -1);
								(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
							}
							(_file_it == NULL) ? NULL : (_file_it = (gee_collection_object_unref (_file_it), NULL));
						}
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) self->priv->_model, 4, GTK_SORT_ASCENDING);
	gtk_tree_sortable_set_sort_func ((GtkTreeSortable*) self->priv->_model, 4, _vtg_project_manager_sort_model_gtk_tree_iter_compare_func, g_object_ref (self), g_object_unref);
}


static void vtg_project_manager_parse_project (VtgProjectManager* self) {
	char* _tmp0;
	char* _tmp7;
	gboolean _tmp8;
	g_return_if_fail (self != NULL);
	/*this.modules.clear ();
	this.groups.clear ();*/
	gee_collection_clear ((GeeCollection*) self->exec_targets);
	gee_collection_clear ((GeeCollection*) self->all_vala_sources);
	_tmp0 = NULL;
	self->changelog_uri = (_tmp0 = NULL, self->changelog_uri = (g_free (self->changelog_uri), NULL), _tmp0);
	{
		GeeList* _tmp1;
		GeeIterator* _tmp2;
		GeeIterator* _group_it;
		_tmp1 = NULL;
		_tmp2 = NULL;
		_group_it = (_tmp2 = gee_iterable_iterator ((GeeIterable*) (_tmp1 = vbf_project_get_groups (self->priv->_project))), (_tmp1 == NULL) ? NULL : (_tmp1 = (gee_collection_object_unref (_tmp1), NULL)), _tmp2);
		while (gee_iterator_next (_group_it)) {
			VbfGroup* group;
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp3;
				GeeIterator* _tmp4;
				GeeIterator* _target_it;
				_tmp3 = NULL;
				_tmp4 = NULL;
				_target_it = (_tmp4 = gee_iterable_iterator ((GeeIterable*) (_tmp3 = vbf_group_get_targets (group))), (_tmp3 == NULL) ? NULL : (_tmp3 = (gee_collection_object_unref (_tmp3), NULL)), _tmp4);
				while (gee_iterator_next (_target_it)) {
					VbfTarget* target;
					target = (VbfTarget*) gee_iterator_get (_target_it);
					if (target->type == VBF_TARGET_TYPES_PROGRAM) {
						gee_collection_add ((GeeCollection*) self->exec_targets, target->name);
					}
					{
						GeeList* _tmp5;
						GeeIterator* _tmp6;
						GeeIterator* _source_it;
						_tmp5 = NULL;
						_tmp6 = NULL;
						_source_it = (_tmp6 = gee_iterable_iterator ((GeeIterable*) (_tmp5 = vbf_target_get_sources (target))), (_tmp5 == NULL) ? NULL : (_tmp5 = (gee_collection_object_unref (_tmp5), NULL)), _tmp6);
						while (gee_iterator_next (_source_it)) {
							VbfSource* source;
							source = (VbfSource*) gee_iterator_get (_source_it);
							if (((VbfFile*) source)->type == VBF_FILE_TYPES_VALA_SOURCE) {
								gee_collection_add ((GeeCollection*) self->all_vala_sources, source);
							}
							(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
						}
						(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	_tmp7 = NULL;
	if ((_tmp8 = g_file_test (_tmp7 = g_build_filename (self->priv->_project->working_dir, "changelog", NULL), G_FILE_TEST_EXISTS), _tmp7 = (g_free (_tmp7), NULL), _tmp8)) {
		char* _tmp10;
		char* _tmp9;
		_tmp10 = NULL;
		_tmp9 = NULL;
		self->changelog_uri = (_tmp10 = g_strdup_printf ("file://%s", _tmp9 = g_build_filename (self->priv->_project->working_dir, "changelog", NULL)), self->changelog_uri = (g_free (self->changelog_uri), NULL), _tmp10);
		_tmp9 = (g_free (_tmp9), NULL);
	} else {
		char* _tmp11;
		gboolean _tmp12;
		_tmp11 = NULL;
		if ((_tmp12 = g_file_test (_tmp11 = g_build_filename (self->priv->_project->working_dir, "ChangeLog", NULL), G_FILE_TEST_EXISTS), _tmp11 = (g_free (_tmp11), NULL), _tmp12)) {
			char* _tmp14;
			char* _tmp13;
			_tmp14 = NULL;
			_tmp13 = NULL;
			self->changelog_uri = (_tmp14 = g_strdup_printf ("file://%s", _tmp13 = g_build_filename (self->priv->_project->working_dir, "ChangeLog", NULL)), self->changelog_uri = (g_free (self->changelog_uri), NULL), _tmp14);
			_tmp13 = (g_free (_tmp13), NULL);
		}
	}
}


static gint vtg_project_manager_sort_model (VtgProjectManager* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b) {
	char* vala;
	char* valb;
	gint _tmp0;
	g_return_val_if_fail (self != NULL, 0);
	g_return_val_if_fail (model != NULL, 0);
	vala = NULL;
	valb = NULL;
	gtk_tree_model_get (model, &(*a), 4, &vala, -1);
	gtk_tree_model_get (model, &(*b), 4, &valb, -1);
	return (_tmp0 = vtg_path_utils_compare_vala_filenames (vala, valb), vala = (g_free (vala), NULL), valb = (g_free (valb), NULL), _tmp0);
}


VtgProjectManager* vtg_project_manager_construct (GType object_type) {
	VtgProjectManager * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VtgProjectManager* vtg_project_manager_new (void) {
	return vtg_project_manager_construct (VTG_TYPE_PROJECT_MANAGER);
}


GtkTreeModel* vtg_project_manager_get_model (VtgProjectManager* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return (GtkTreeModel*) self->priv->_model;
}


VbfProject* vtg_project_manager_get_project (VtgProjectManager* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_project;
}


static void vtg_project_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgProjectManager * self;
	gpointer boxed;
	self = VTG_PROJECT_MANAGER (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_MODEL:
		g_value_set_object (value, vtg_project_manager_get_model (self));
		break;
		case VTG_PROJECT_MANAGER_PROJECT:
		g_value_set_object (value, vtg_project_manager_get_project (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_class_init (VtgProjectManagerClass * klass) {
	vtg_project_manager_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectManagerPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_project_manager_get_property;
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_MODEL, g_param_spec_object ("model", "model", "model", GTK_TYPE_TREE_MODEL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_PROJECT, g_param_spec_object ("project", "project", "project", VBF_TYPE_PROJECT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_signal_new ("updated", VTG_TYPE_PROJECT_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void vtg_project_manager_instance_init (VtgProjectManager * self) {
	self->priv = VTG_PROJECT_MANAGER_GET_PRIVATE (self);
	self->priv->_project = NULL;
	self->priv->in_update = FALSE;
	self->filename = NULL;
	self->exec_targets = (GeeList*) gee_array_list_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, g_direct_equal);
	self->all_vala_sources = (GeeList*) gee_array_list_new (VBF_TYPE_SOURCE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->vcs_type = VTG_VCS_TYPES_NONE;
	self->changelog_uri = NULL;
}


static void vtg_project_manager_finalize (GObject* obj) {
	VtgProjectManager * self;
	self = VTG_PROJECT_MANAGER (obj);
	{
	}
	(self->priv->_project == NULL) ? NULL : (self->priv->_project = (g_object_unref (self->priv->_project), NULL));
	(self->priv->_model == NULL) ? NULL : (self->priv->_model = (g_object_unref (self->priv->_model), NULL));
	self->filename = (g_free (self->filename), NULL);
	(self->exec_targets == NULL) ? NULL : (self->exec_targets = (gee_collection_object_unref (self->exec_targets), NULL));
	(self->all_vala_sources == NULL) ? NULL : (self->all_vala_sources = (gee_collection_object_unref (self->all_vala_sources), NULL));
	self->changelog_uri = (g_free (self->changelog_uri), NULL);
	G_OBJECT_CLASS (vtg_project_manager_parent_class)->finalize (obj);
}


static GType vtg_project_manager_type_id = 0;
GType vtg_project_manager_get_type (void) {
	return vtg_project_manager_type_id;
}


GType vtg_project_manager_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManager), 0, (GInstanceInitFunc) vtg_project_manager_instance_init, NULL };
	vtg_project_manager_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManager", &g_define_type_info, 0);
	return vtg_project_manager_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




