/*
 *  vtgprojectmanager.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#ifndef __VTGPROJECTMANAGER_H__
#define __VTGPROJECTMANAGER_H__

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee/list.h>
#include <vbfsource.h>
#include <gtk/gtk.h>
#include <vbfproject.h>
#include "vtgutils.h"

G_BEGIN_DECLS


#define VTG_TYPE_PROJECT_MANAGER (vtg_project_manager_get_type ())
#define VTG_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_MANAGER, VtgProjectManager))
#define VTG_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerClass))
#define VTG_IS_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_MANAGER))
#define VTG_IS_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_MANAGER))
#define VTG_PROJECT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerClass))

typedef struct _VtgProjectManager VtgProjectManager;
typedef struct _VtgProjectManagerClass VtgProjectManagerClass;
typedef struct _VtgProjectManagerPrivate VtgProjectManagerPrivate;

struct _VtgProjectManager {
	GObject parent_instance;
	VtgProjectManagerPrivate * priv;
	char* filename;
	GeeList* exec_targets;
	GeeList* all_vala_sources;
	VtgVcsTypes vcs_type;
	char* changelog_uri;
};

struct _VtgProjectManagerClass {
	GObjectClass parent_class;
};


gboolean vtg_project_manager_contains_file (VtgProjectManager* self, const char* uri);
VbfSource* vtg_project_manager_get_source_file_from_uri (VtgProjectManager* self, const char* uri);
gboolean vtg_project_manager_contains_vala_source_file (VtgProjectManager* self, const char* uri);
char* vtg_project_manager_source_uri_for_name (VtgProjectManager* self, const char* name);
gboolean vtg_project_manager_open (VtgProjectManager* self, const char* project_filename, GError** error);
void vtg_project_manager_close (VtgProjectManager* self);
VtgProjectManager* vtg_project_manager_construct (GType object_type);
VtgProjectManager* vtg_project_manager_new (void);
GtkTreeModel* vtg_project_manager_get_model (VtgProjectManager* self);
VbfProject* vtg_project_manager_get_project (VtgProjectManager* self);
GType vtg_project_manager_get_type (void);
GType vtg_project_manager_register_type (GTypeModule * module);


G_END_DECLS

#endif
