/*
 *  vtgsourcebookmarks.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgsourcebookmarks.h"
#include <gee/arraylist.h>
#include <gee/list.h>
#include <gedit/gedit-window.h>
#include <gedit/gedit-tab.h>
#include <gedit/gedit-document.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gee/collection.h>
#include "vtgprojectmanagerui.h"
#include "vtgprojectview.h"
#include "vtgprojectmanager.h"




struct _VtgSourceBookmarksPrivate {
	VtgPluginInstance* _plugin_instance;
	gulong signal_id;
	GeeList* _bookmarks;
	gint _current_bookmark_index;
};

#define VTG_SOURCE_BOOKMARKS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_SOURCE_BOOKMARKS, VtgSourceBookmarksPrivate))
enum  {
	VTG_SOURCE_BOOKMARKS_DUMMY_PROPERTY,
	VTG_SOURCE_BOOKMARKS_PLUGIN_INSTANCE,
	VTG_SOURCE_BOOKMARKS_IS_EMPTY
};
#define VTG_SOURCE_BOOKMARKS_MAX_BOOKMARKS 20
static void vtg_source_bookmarks_on_tab_changed (GeditWindow* sender, GeditTab* tab, VtgSourceBookmarks* instance);
static void vtg_source_bookmarks_set_plugin_instance (VtgSourceBookmarks* self, VtgPluginInstance* value);
static gpointer vtg_source_bookmarks_parent_class = NULL;
static void vtg_source_bookmarks_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



VtgSourceBookmarks* vtg_source_bookmarks_construct (GType object_type, VtgPluginInstance* plugin_instance) {
	GParameter * __params;
	GParameter * __params_it;
	VtgSourceBookmarks * self;
	GeditWindow* _tmp0;
	GeditWindow* window;
	g_return_val_if_fail (plugin_instance != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "plugin-instance";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN_INSTANCE);
	g_value_set_object (&__params_it->value, plugin_instance);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	_tmp0 = NULL;
	window = (_tmp0 = vtg_plugin_instance_get_window (self->priv->_plugin_instance), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	self->priv->signal_id = g_signal_connect (window, "active_tab_changed", (GCallback) vtg_source_bookmarks_on_tab_changed, self);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgSourceBookmarks* vtg_source_bookmarks_new (VtgPluginInstance* plugin_instance) {
	return vtg_source_bookmarks_construct (VTG_TYPE_SOURCE_BOOKMARKS, plugin_instance);
}


static void vtg_source_bookmarks_on_tab_changed (GeditWindow* sender, GeditTab* tab, VtgSourceBookmarks* instance) {
	GeditDocument* _tmp0;
	GeditDocument* doc;
	GtkTextMark* mark;
	GtkTextIter start = {0};
	const char* _tmp1;
	char* uri;
	VtgProjectManager* _tmp2;
	VtgProjectManager* prj;
	gboolean _tmp3;
	g_return_if_fail (sender != NULL);
	g_return_if_fail (tab != NULL);
	g_return_if_fail (instance != NULL);
	_tmp0 = NULL;
	doc = (_tmp0 = gedit_tab_get_document (tab), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert ((GtkTextBuffer*) doc));
	gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) doc, &start, mark);
	_tmp1 = NULL;
	uri = (_tmp1 = gedit_document_get_uri (doc), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
	_tmp2 = NULL;
	prj = (_tmp2 = vtg_project_view_get_current_project (vtg_project_manager_ui_get_project_view (vtg_plugin_instance_get_project_manager_ui (instance->priv->_plugin_instance))), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
	_tmp3 = FALSE;
	if (prj != NULL) {
		_tmp3 = vtg_project_manager_contains_vala_source_file (prj, uri);
	} else {
		_tmp3 = FALSE;
	}
	if (_tmp3) {
		gint line;
		gint col;
		VtgSourceBookmark* book;
		char* _tmp5;
		const char* _tmp4;
		line = gtk_text_iter_get_line (&start);
		col = gtk_text_iter_get_line_offset (&start);
		book = vtg_source_bookmark_new ();
		_tmp5 = NULL;
		_tmp4 = NULL;
		book->uri = (_tmp5 = (_tmp4 = uri, (_tmp4 == NULL) ? NULL : g_strdup (_tmp4)), book->uri = (g_free (book->uri), NULL), _tmp5);
		book->line = line;
		book->column = col;
		vtg_source_bookmarks_add_bookmark (instance, book);
		(book == NULL) ? NULL : (book = (vtg_source_bookmark_unref (book), NULL));
	}
	(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
	uri = (g_free (uri), NULL);
	(prj == NULL) ? NULL : (prj = (g_object_unref (prj), NULL));
}


/*
private void debug_dump_list ()
{
print ("DUMP Bookmarks:\n");
if (_bookmarks.size == 0) {
print ("   the bookmark list is empty");
} else {
int idx = 0;
foreach (SourceBookmark item in _bookmarks) {
print ("   %d: %s - %d,%d\n", idx, item.uri, item.line, item.column);
idx++;
}
print ("   current index is %d\n", _current_bookmark_index);
}
}
*/
void vtg_source_bookmarks_add_bookmark (VtgSourceBookmarks* self, VtgSourceBookmark* item) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (item != NULL);
	if (gee_collection_get_size ((GeeCollection*) self->priv->_bookmarks) > 0) {
		gint index;
		VtgSourceBookmark* prev;
		index = self->priv->_current_bookmark_index;
		if (index == (-1)) {
			index = gee_collection_get_size ((GeeCollection*) self->priv->_bookmarks) - 1;
		}
		prev = (VtgSourceBookmark*) gee_list_get (self->priv->_bookmarks, index);
		if (_vala_strcmp0 (prev->uri, item->uri) == 0) {
			prev->line = item->line;
			prev->column = item->column;
			(prev == NULL) ? NULL : (prev = (vtg_source_bookmark_unref (prev), NULL));
			return;
		}
		(prev == NULL) ? NULL : (prev = (vtg_source_bookmark_unref (prev), NULL));
	}
	/*avoid duplicate item*/
	if (gee_collection_get_size ((GeeCollection*) self->priv->_bookmarks) == VTG_SOURCE_BOOKMARKS_MAX_BOOKMARKS) {
		gee_list_remove_at (self->priv->_bookmarks, 0);
	}
	if (self->priv->_current_bookmark_index >= (gee_collection_get_size ((GeeCollection*) self->priv->_bookmarks) - 1)) {
		gee_collection_add ((GeeCollection*) self->priv->_bookmarks, item);
		self->priv->_current_bookmark_index = gee_collection_get_size ((GeeCollection*) self->priv->_bookmarks) - 1;
	} else {
		self->priv->_current_bookmark_index++;
		gee_list_insert (self->priv->_bookmarks, self->priv->_current_bookmark_index, item);
	}
}


/*debug_dump_list ();*/
void vtg_source_bookmarks_remove_current_bookmark (VtgSourceBookmarks* self) {
	VtgSourceBookmark* item;
	g_return_if_fail (self != NULL);
	item = vtg_source_bookmarks_get_current_bookmark (self);
	if (item != NULL) {
		gee_collection_remove ((GeeCollection*) self->priv->_bookmarks, item);
		self->priv->_current_bookmark_index = gee_collection_get_size ((GeeCollection*) self->priv->_bookmarks) - 1;
	}
	(item == NULL) ? NULL : (item = (vtg_source_bookmark_unref (item), NULL));
}


VtgSourceBookmark* vtg_source_bookmarks_get_current_bookmark (VtgSourceBookmarks* self) {
	VtgSourceBookmark* item;
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, NULL);
	item = NULL;
	_tmp0 = FALSE;
	if (gee_collection_get_size ((GeeCollection*) self->priv->_bookmarks) > 0) {
		_tmp0 = gee_collection_get_size ((GeeCollection*) self->priv->_bookmarks) > self->priv->_current_bookmark_index;
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		VtgSourceBookmark* _tmp1;
		_tmp1 = NULL;
		item = (_tmp1 = (VtgSourceBookmark*) gee_list_get (self->priv->_bookmarks, self->priv->_current_bookmark_index), (item == NULL) ? NULL : (item = (vtg_source_bookmark_unref (item), NULL)), _tmp1);
	}
	return item;
}


void vtg_source_bookmarks_move_next (VtgSourceBookmarks* self) {
	gboolean wrap;
	g_return_if_fail (self != NULL);
	wrap = FALSE;
	if (gee_collection_get_size ((GeeCollection*) self->priv->_bookmarks) == 0) {
		return;
	}
	if (self->priv->_current_bookmark_index < (gee_collection_get_size ((GeeCollection*) self->priv->_bookmarks) - 1)) {
		self->priv->_current_bookmark_index++;
	} else {
		self->priv->_current_bookmark_index = 0;
		wrap = TRUE;
	}
	g_signal_emit_by_name (self, "current-bookmark-changed");
	if (wrap) {
		g_signal_emit_by_name (self, "move-wrapped");
	}
}


void vtg_source_bookmarks_move_previous (VtgSourceBookmarks* self) {
	gboolean wrap;
	g_return_if_fail (self != NULL);
	wrap = FALSE;
	if (gee_collection_get_size ((GeeCollection*) self->priv->_bookmarks) == 0) {
		return;
	}
	if (self->priv->_current_bookmark_index > 0) {
		self->priv->_current_bookmark_index--;
	} else {
		self->priv->_current_bookmark_index = gee_collection_get_size ((GeeCollection*) self->priv->_bookmarks) - 1;
		wrap = TRUE;
	}
	g_signal_emit_by_name (self, "current-bookmark-changed");
	if (wrap) {
		g_signal_emit_by_name (self, "move-wrapped");
	}
}


VtgPluginInstance* vtg_source_bookmarks_get_plugin_instance (VtgSourceBookmarks* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_plugin_instance;
}


static void vtg_source_bookmarks_set_plugin_instance (VtgSourceBookmarks* self, VtgPluginInstance* value) {
	VtgPluginInstance* _tmp2;
	VtgPluginInstance* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_plugin_instance = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_plugin_instance == NULL) ? NULL : (self->priv->_plugin_instance = (g_object_unref (self->priv->_plugin_instance), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "plugin-instance");
}


gboolean vtg_source_bookmarks_get_is_empty (VtgSourceBookmarks* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return gee_collection_get_size ((GeeCollection*) self->priv->_bookmarks) == 0;
}


static void vtg_source_bookmarks_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgSourceBookmarks * self;
	gpointer boxed;
	self = VTG_SOURCE_BOOKMARKS (object);
	switch (property_id) {
		case VTG_SOURCE_BOOKMARKS_PLUGIN_INSTANCE:
		g_value_set_object (value, vtg_source_bookmarks_get_plugin_instance (self));
		break;
		case VTG_SOURCE_BOOKMARKS_IS_EMPTY:
		g_value_set_boolean (value, vtg_source_bookmarks_get_is_empty (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_source_bookmarks_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgSourceBookmarks * self;
	self = VTG_SOURCE_BOOKMARKS (object);
	switch (property_id) {
		case VTG_SOURCE_BOOKMARKS_PLUGIN_INSTANCE:
		vtg_source_bookmarks_set_plugin_instance (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_source_bookmarks_class_init (VtgSourceBookmarksClass * klass) {
	vtg_source_bookmarks_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgSourceBookmarksPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_source_bookmarks_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_source_bookmarks_set_property;
	G_OBJECT_CLASS (klass)->finalize = vtg_source_bookmarks_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SOURCE_BOOKMARKS_PLUGIN_INSTANCE, g_param_spec_object ("plugin-instance", "plugin-instance", "plugin-instance", VTG_TYPE_PLUGIN_INSTANCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SOURCE_BOOKMARKS_IS_EMPTY, g_param_spec_boolean ("is-empty", "is-empty", "is-empty", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_signal_new ("current_bookmark_changed", VTG_TYPE_SOURCE_BOOKMARKS, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("move_wrapped", VTG_TYPE_SOURCE_BOOKMARKS, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void vtg_source_bookmarks_instance_init (VtgSourceBookmarks * self) {
	self->priv = VTG_SOURCE_BOOKMARKS_GET_PRIVATE (self);
	self->priv->_bookmarks = (GeeList*) gee_array_list_new (VTG_TYPE_SOURCE_BOOKMARK, (GBoxedCopyFunc) vtg_source_bookmark_ref, vtg_source_bookmark_unref, g_direct_equal);
	self->priv->_current_bookmark_index = -1;
}


static void vtg_source_bookmarks_finalize (GObject* obj) {
	VtgSourceBookmarks * self;
	self = VTG_SOURCE_BOOKMARKS (obj);
	{
		g_signal_handler_disconnect (self, self->priv->signal_id);
	}
	(self->priv->_plugin_instance == NULL) ? NULL : (self->priv->_plugin_instance = (g_object_unref (self->priv->_plugin_instance), NULL));
	(self->priv->_bookmarks == NULL) ? NULL : (self->priv->_bookmarks = (gee_collection_object_unref (self->priv->_bookmarks), NULL));
	G_OBJECT_CLASS (vtg_source_bookmarks_parent_class)->finalize (obj);
}


static GType vtg_source_bookmarks_type_id = 0;
GType vtg_source_bookmarks_get_type (void) {
	return vtg_source_bookmarks_type_id;
}


GType vtg_source_bookmarks_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgSourceBookmarksClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_source_bookmarks_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgSourceBookmarks), 0, (GInstanceInitFunc) vtg_source_bookmarks_instance_init, NULL };
	vtg_source_bookmarks_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgSourceBookmarks", &g_define_type_info, 0);
	return vtg_source_bookmarks_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




